@@ -571,6 +571,22 @@ failed:
     return DIERR_OUTOFMEMORY;
 }
 
+static int verify_offset(const DataFormat *df, int offset)
+{
+    int i;
+
+    if (!df->offsets)
+        return -1;
+
+    for (i = df->wine_df->dwNumObjs - 1; i >= 0; i--)
+    {
+        if (df->offsets[i] == offset)
+            return offset;
+    }
+
+    return -1;
+}
+
 /* find an object by its offset in a data format */
 static int offset_to_object(const DataFormat *df, int offset)
 {
@@ -759,6 +775,46 @@ static BOOL load_mapping_settings(IDirectInputDeviceImpl *This, LPDIACTIONFORMAT
     return mapped > 0;
 }
 
+static BOOL set_app_data(IDirectInputDeviceImpl *dev, int offset, UINT_PTR app_data)
+{
+    int num_actions = dev->num_actions;
+    ActionMap *action_map = dev->action_map, *target_map = NULL;
+
+    if (num_actions == 0)
+    {
+        num_actions = 1;
+        action_map = HeapAlloc(GetProcessHeap(), 0, sizeof(ActionMap));
+        if (!action_map) return FALSE;
+        target_map = &action_map[0];
+    }
+    else
+    {
+        int i;
+        for (i = 0; i < num_actions; i++)
+        {
+            if (dev->action_map[i].offset != offset) continue;
+            target_map = &dev->action_map[i];
+            break;
+        }
+
+        if (!target_map)
+        {
+            num_actions++;
+            action_map = HeapReAlloc(GetProcessHeap(), 0, action_map, sizeof(ActionMap)*num_actions);
+            if (!action_map) return FALSE;
+            target_map = &action_map[num_actions-1];
+        }
+    }
+
+    target_map->offset = offset;
+    target_map->uAppData = app_data;
+
+    dev->action_map = action_map;
+    dev->num_actions = num_actions;
+
+    return TRUE;
+}
+
 HRESULT _build_action_map(LPDIRECTINPUTDEVICE8W iface, LPDIACTIONFORMATW lpdiaf, LPCWSTR lpszUserName, DWORD dwFlags, DWORD devMask, LPCDIDATAFORMAT df)
 {
     IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
@@ -833,6 +889,7 @@ HRESULT _set_action_map(LPDIRECTINPUTDEVICE8W iface, LPDIACTIONFORMATW lpdiaf, L
     DWORD username_size = MAX_PATH;
     int i, action = 0, num_actions = 0;
     unsigned int offset = 0;
+    ActionMap *action_map;
 
     if (This->acquired) return DIERR_ACQUIRED;
 
@@ -848,15 +905,12 @@ HRESULT _set_action_map(LPDIRECTINPUTDEVICE8W iface, LPDIACTIONFORMATW lpdiaf, L
 
     if (num_actions == 0) return DI_NOEFFECT;
 
-    This->num_actions = num_actions;
-
     /* Construct the dataformat and actionmap */
     obj_df = HeapAlloc(GetProcessHeap(), 0, sizeof(DIOBJECTDATAFORMAT)*num_actions);
     data_format.rgodf = (LPDIOBJECTDATAFORMAT)obj_df;
     data_format.dwNumObjs = num_actions;
 
-    HeapFree(GetProcessHeap(), 0, This->action_map);
-    This->action_map = HeapAlloc(GetProcessHeap(), 0, sizeof(ActionMap)*num_actions);
+    action_map = HeapAlloc(GetProcessHeap(), 0, sizeof(ActionMap)*num_actions);
 
     for (i = 0; i < lpdiaf->dwNumActions; i++)
     {
@@ -873,8 +927,8 @@ HRESULT _set_action_map(LPDIRECTINPUTDEVICE8W iface, LPDIACTIONFORMATW lpdiaf, L
 
             memcpy(&obj_df[action], obj, df->dwObjSize);
 
-            This->action_map[action].uAppData = lpdiaf->rgoAction[i].uAppData;
-            This->action_map[action].offset = offset;
+            action_map[action].uAppData = lpdiaf->rgoAction[i].uAppData;
+            action_map[action].offset = offset;
             obj_df[action].dwOfs = offset;
             offset += (type & DIDFT_BUTTON) ? 1 : 4;
 
@@ -884,6 +938,9 @@ HRESULT _set_action_map(LPDIRECTINPUTDEVICE8W iface, LPDIACTIONFORMATW lpdiaf, L
 
     IDirectInputDevice8_SetDataFormat(iface, &data_format);
 
+    This->action_map = action_map;
+    This->num_actions = num_actions;
+
     HeapFree(GetProcessHeap(), 0, obj_df);
 
     /* Set the device properties according to the action format */
@@ -962,6 +1019,7 @@ void queue_event(LPDIRECTINPUTDEVICE8A iface, int inst_id, DWORD data, DWORD tim
     This->data_queue[This->queue_head].dwData      = data;
     This->data_queue[This->queue_head].dwTimeStamp = time;
     This->data_queue[This->queue_head].dwSequence  = seq;
+    This->data_queue[This->queue_head].uAppData    = -1;
 
     /* Set uAppData by means of action mapping */
     if (This->num_actions > 0)
@@ -1063,6 +1121,10 @@ HRESULT WINAPI IDirectInputDevice2WImpl_SetDataFormat(LPDIRECTINPUTDEVICE8W ifac
 
     EnterCriticalSection(&This->crit);
 
+    HeapFree(GetProcessHeap(), 0, This->action_map);
+    This->action_map = NULL;
+    This->num_actions = 0;
+
     release_DataFormat(&This->data_format);
     res = create_DataFormat(df, &This->data_format);
 
@@ -1441,6 +1503,23 @@ HRESULT WINAPI IDirectInputDevice2WImpl_SetProperty(
                 lstrcpynW(device_player->username, ps->wsz, ARRAY_SIZE(device_player->username));
             break;
         }
+        case (DWORD_PTR) DIPROP_APPDATA:
+        {
+            int offset = -1;
+            LPCDIPROPPOINTER pp = (LPCDIPROPPOINTER)pdiph;
+            if (pdiph->dwSize != sizeof(DIPROPPOINTER)) return DIERR_INVALIDPARAM;
+
+            if (pdiph->dwHow == DIPH_BYID)
+                offset = id_to_offset(&This->data_format, pdiph->dwObj);
+            else if (pdiph->dwHow == DIPH_BYOFFSET)
+                offset = verify_offset(&This->data_format, pdiph->dwObj);
+            else
+                return DIERR_UNSUPPORTED;
+
+            if (offset == -1) return DIERR_OBJECTNOTFOUND;
+            if (!set_app_data(This, offset, pp->uData)) return DIERR_OUTOFMEMORY;
+            break;
+        }
         default:
             WARN("Unknown property %s\n", debugstr_guid(rguid));
             return DIERR_UNSUPPORTED;
