@@ -54,7 +54,7 @@ typedef struct {
     } u;
 } exprval_t;
 
-static const size_t stack_size = 0x4000;
+static const size_t stack_size = 0x40000;
 
 static HRESULT stack_push(script_ctx_t *ctx, jsval_t v)
 {
@@ -1304,7 +1304,7 @@ static HRESULT interp_local_ref(script_ctx_t *ctx)
     call_frame_t *frame = ctx->call_ctx;
     exprval_t ref;
 
-    TRACE("%d\n", arg);
+    TRACE("%s\n", debugstr_w(local_name(frame, arg)));
 
     if(!frame->base_scope || !frame->base_scope->frame)
         return interp_identifier_ref(ctx, local_name(frame, arg), flags);
@@ -1321,15 +1321,16 @@ static HRESULT interp_local(script_ctx_t *ctx)
     jsval_t copy;
     HRESULT hres;
 
-    TRACE("%d: %s\n", arg, debugstr_w(local_name(frame, arg)));
-
-    if(!frame->base_scope || !frame->base_scope->frame)
+    if(!frame->base_scope || !frame->base_scope->frame) {
+        TRACE("%s\n", debugstr_w(local_name(frame, arg)));
         return identifier_value(ctx, local_name(frame, arg));
+    }
 
     hres = jsval_copy(ctx->stack[local_off(frame, arg)], &copy);
     if(FAILED(hres))
         return hres;
 
+    TRACE("%s: %s\n", debugstr_w(local_name(frame, arg)), debugstr_jsval(copy));
     return stack_push(ctx, copy);
 }
 
@@ -2510,6 +2511,25 @@ static HRESULT interp_rshift2(script_ctx_t *ctx)
     return stack_push(ctx, jsval_number(l >> (r&0x1f)));
 }
 
+/* ECMA-262 3rd Edition    9.8 */
+static HRESULT interp_to_string(script_ctx_t *ctx)
+{
+    jsstr_t *str;
+    jsval_t v;
+    HRESULT hres;
+
+    v = stack_pop(ctx);
+    TRACE("%s\n", debugstr_jsval(v));
+    hres = to_string(ctx, v, &str);
+    jsval_release(v);
+    if(FAILED(hres)) {
+        WARN("failed %08x\n", hres);
+        return hres;
+    }
+
+    return stack_push(ctx, jsval_string(str));
+}
+
 /* ECMA-262 3rd Edition    11.13.1 */
 static HRESULT interp_assign(script_ctx_t *ctx)
 {
@@ -2536,6 +2556,41 @@ static HRESULT interp_assign(script_ctx_t *ctx)
     return stack_push(ctx, v);
 }
 
+/* ECMA-262 3rd Edition    11.13.1 */
+static HRESULT interp_set_member(script_ctx_t *ctx)
+{
+    jsval_t objv, namev, value;
+    const WCHAR *name;
+    IDispatch *obj;
+    HRESULT hres;
+
+    value = stack_pop(ctx);
+    namev = stack_pop(ctx);
+    assert(is_string(namev));
+    objv = stack_pop(ctx);
+
+    TRACE("%s.%s = %s\n", debugstr_jsval(objv), debugstr_jsval(namev), debugstr_jsval(value));
+
+    hres = to_object(ctx, objv, &obj);
+    jsval_release(objv);
+    if(SUCCEEDED(hres) && !(name = jsstr_flatten(get_string(namev)))) {
+        IDispatch_Release(obj);
+        hres = E_OUTOFMEMORY;
+    }
+    if(SUCCEEDED(hres)) {
+        hres = disp_propput_name(ctx, obj, name, value);
+        IDispatch_Release(obj);
+        jsstr_release(get_string(namev));
+    }
+    if(FAILED(hres)) {
+        WARN("failed %08x\n", hres);
+        jsval_release(value);
+        return hres;
+    }
+
+    return stack_push(ctx, value);
+}
+
 /* JScript extension */
 static HRESULT interp_assign_call(script_ctx_t *ctx)
 {
@@ -2703,7 +2758,7 @@ static void pop_call_frame(script_ctx_t *ctx)
 
 static void print_backtrace(script_ctx_t *ctx)
 {
-    unsigned depth = 0, i;
+    unsigned depth = 0, i, line, char_pos;
     call_frame_t *frame;
 
     for(frame = ctx->call_ctx; frame; frame = frame->prev_frame) {
@@ -2724,7 +2779,8 @@ static void print_backtrace(script_ctx_t *ctx)
         }else {
             WARN("[detached frame]");
         }
-        WARN(")\n");
+        line = get_location_line(frame->bytecode, frame->bytecode->instrs[frame->ip].loc, &char_pos);
+        WARN(") context %s line %u char %u\n", wine_dbgstr_longlong(frame->bytecode->source_context), line, char_pos);
 
         if(!(frame->flags & EXEC_RETURN_TO_INTERP)) {
             WARN("%u\t[native code]\n", depth);
