@@ -38,7 +38,6 @@
 #include "dxva2api.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 #include "wine/list.h"
 
 #include "mf_private.h"
@@ -51,8 +50,6 @@ BOOL WINAPI DllMain(HINSTANCE instance, DWORD reason, LPVOID reserved)
 {
     switch (reason)
     {
-        case DLL_WINE_PREATTACH:
-            return FALSE;    /* prefer native version */
         case DLL_PROCESS_ATTACH:
             mfinstance = instance;
             DisableThreadLibraryCalls(instance);
@@ -107,11 +104,17 @@ enum media_stream_flags
     STREAM_FLAG_PRESENTED = 0x4,        /* Set if stream was selected last time Start() was called. */
 };
 
+struct stream_transform
+{
+    IMFTransform *transform;
+    unsigned int min_buffer_size;
+};
+
 struct media_stream
 {
     IMFMediaStream *stream;
     IMFMediaType *current;
-    IMFTransform *decoder;
+    struct stream_transform decoder;
     IMFVideoSampleAllocatorEx *allocator;
     IMFVideoSampleAllocatorNotify notify_cb;
     unsigned int id;
@@ -191,6 +194,7 @@ struct source_reader
     unsigned int last_read_index;
     unsigned int stream_count;
     unsigned int flags;
+    unsigned int queue;
     enum media_source_state source_state;
     struct media_stream *streams;
     struct list responses;
@@ -258,7 +262,7 @@ static ULONG WINAPI source_reader_async_command_Release(IUnknown *iface)
     {
         if (command->op == SOURCE_READER_ASYNC_SEEK)
             PropVariantClear(&command->u.seek.position);
-        heap_free(command);
+        free(command);
     }
 
     return refcount;
@@ -275,7 +279,7 @@ static HRESULT source_reader_create_async_op(enum source_reader_async_op op, str
 {
     struct source_reader_async_command *command;
 
-    if (!(command = heap_alloc_zero(sizeof(*command))))
+    if (!(command = calloc(1, sizeof(*command))))
         return E_OUTOFMEMORY;
 
     command->IUnknown_iface.lpVtbl = &source_reader_async_command_vtbl;
@@ -298,14 +302,14 @@ static HRESULT media_event_get_object(IMFMediaEvent *event, REFIID riid, void **
         return hr;
     }
 
-    if (value.vt != VT_UNKNOWN || !value.u.punkVal)
+    if (value.vt != VT_UNKNOWN || !value.punkVal)
     {
         WARN("Unexpected value type %d.\n", value.vt);
         PropVariantClear(&value);
         return E_UNEXPECTED;
     }
 
-    hr = IUnknown_QueryInterface(value.u.punkVal, riid, obj);
+    hr = IUnknown_QueryInterface(value.punkVal, riid, obj);
     PropVariantClear(&value);
     if (FAILED(hr))
     {
@@ -380,8 +384,7 @@ static void source_reader_response_ready(struct source_reader *reader, struct st
         if (SUCCEEDED(source_reader_create_async_op(SOURCE_READER_ASYNC_SAMPLE_READY, &command)))
         {
             command->u.sample.stream_index = stream->index;
-            if (FAILED(hr = MFPutWorkItem(MFASYNC_CALLBACK_QUEUE_STANDARD, &reader->async_commands_callback,
-                    &command->IUnknown_iface)))
+            if (FAILED(hr = MFPutWorkItem(reader->queue, &reader->async_commands_callback, &command->IUnknown_iface)))
                 WARN("Failed to submit async result, hr %#x.\n", hr);
             IUnknown_Release(&command->IUnknown_iface);
         }
@@ -444,7 +447,7 @@ static void source_reader_queue_response(struct source_reader *reader, struct me
 {
     struct stream_response *response;
 
-    response = heap_alloc_zero(sizeof(*response));
+    response = calloc(1, sizeof(*response));
     response->status = status;
     response->stream_index = stream->index;
     response->stream_flags = stream_flags;
@@ -546,9 +549,11 @@ static HRESULT source_reader_source_state_handler(struct source_reader *reader,
     {
         case MESourceStarted:
             reader->source_state = SOURCE_STATE_STARTED;
+            reader->flags &= ~SOURCE_READER_SEEKING;
             break;
         case MESourceStopped:
             reader->source_state = SOURCE_STATE_STOPPED;
+            reader->flags &= ~SOURCE_READER_SEEKING;
             break;
         case MESourceSeeked:
             reader->flags &= ~SOURCE_READER_SEEKING;
@@ -645,12 +650,13 @@ static HRESULT source_reader_pull_stream_samples(struct source_reader *reader, s
 {
     MFT_OUTPUT_STREAM_INFO stream_info = { 0 };
     MFT_OUTPUT_DATA_BUFFER out_buffer;
+    unsigned int buffer_size;
     IMFMediaBuffer *buffer;
     LONGLONG timestamp;
     DWORD status;
     HRESULT hr;
 
-    if (FAILED(hr = IMFTransform_GetOutputStreamInfo(stream->decoder, 0, &stream_info)))
+    if (FAILED(hr = IMFTransform_GetOutputStreamInfo(stream->decoder.transform, 0, &stream_info)))
     {
         WARN("Failed to get output stream info, hr %#x.\n", hr);
         return hr;
@@ -660,12 +666,14 @@ static HRESULT source_reader_pull_stream_samples(struct source_reader *reader, s
     {
         memset(&out_buffer, 0, sizeof(out_buffer));
 
-        if (!(stream_info.dwFlags & MFT_OUTPUT_STREAM_PROVIDES_SAMPLES))
+        if (!(stream_info.dwFlags & (MFT_OUTPUT_STREAM_PROVIDES_SAMPLES | MFT_OUTPUT_STREAM_CAN_PROVIDE_SAMPLES)))
         {
             if (FAILED(hr = MFCreateSample(&out_buffer.pSample)))
                 break;
 
-            if (FAILED(hr = MFCreateAlignedMemoryBuffer(stream_info.cbSize, stream_info.cbAlignment, &buffer)))
+            buffer_size = max(stream_info.cbSize, stream->decoder.min_buffer_size);
+
+            if (FAILED(hr = MFCreateAlignedMemoryBuffer(buffer_size, stream_info.cbAlignment, &buffer)))
             {
                 IMFSample_Release(out_buffer.pSample);
                 break;
@@ -675,7 +683,7 @@ static HRESULT source_reader_pull_stream_samples(struct source_reader *reader, s
             IMFMediaBuffer_Release(buffer);
         }
 
-        if (FAILED(hr = IMFTransform_ProcessOutput(stream->decoder, 0, 1, &out_buffer, &status)))
+        if (FAILED(hr = IMFTransform_ProcessOutput(stream->decoder.transform, 0, 1, &out_buffer, &status)))
         {
             if (out_buffer.pSample)
                 IMFSample_Release(out_buffer.pSample);
@@ -702,7 +710,7 @@ static HRESULT source_reader_process_sample(struct source_reader *reader, struct
     LONGLONG timestamp;
     HRESULT hr;
 
-    if (!stream->decoder)
+    if (!stream->decoder.transform)
     {
         timestamp = 0;
         if (FAILED(IMFSample_GetSampleTime(sample, &timestamp)))
@@ -717,7 +725,7 @@ static HRESULT source_reader_process_sample(struct source_reader *reader, struct
     hr = source_reader_pull_stream_samples(reader, stream);
     if (hr == MF_E_TRANSFORM_NEED_MORE_INPUT)
     {
-        if (FAILED(hr = IMFTransform_ProcessInput(stream->decoder, 0, sample, 0)))
+        if (FAILED(hr = IMFTransform_ProcessInput(stream->decoder.transform, 0, sample, 0)))
         {
             WARN("Transform failed to process input, hr %#x.\n", hr);
             return hr;
@@ -814,7 +822,7 @@ static HRESULT source_reader_media_stream_state_handler(struct source_reader *re
                     stream->state = STREAM_STATE_EOS;
                     stream->flags &= ~STREAM_FLAG_SAMPLE_REQUESTED;
 
-                    if (stream->decoder && SUCCEEDED(IMFTransform_ProcessMessage(stream->decoder,
+                    if (stream->decoder.transform && SUCCEEDED(IMFTransform_ProcessMessage(stream->decoder.transform,
                             MFT_MESSAGE_COMMAND_DRAIN, 0)))
                     {
                         if ((hr = source_reader_pull_stream_samples(reader, stream)) != MF_E_TRANSFORM_NEED_MORE_INPUT)
@@ -832,7 +840,7 @@ static HRESULT source_reader_media_stream_state_handler(struct source_reader *re
                 case MEStreamTick:
                     value.vt = VT_EMPTY;
                     hr = SUCCEEDED(IMFMediaEvent_GetValue(event, &value)) && value.vt == VT_I8 ? S_OK : E_UNEXPECTED;
-                    timestamp = SUCCEEDED(hr) ? value.u.hVal.QuadPart : 0;
+                    timestamp = SUCCEEDED(hr) ? value.hVal.QuadPart : 0;
                     PropVariantClear(&value);
 
                     source_reader_queue_response(reader, stream, hr, MF_SOURCE_READERF_STREAMTICK, timestamp, NULL);
@@ -964,7 +972,7 @@ static void source_reader_release_response(struct stream_response *response)
 {
     if (response->sample)
         IMFSample_Release(response->sample);
-    heap_free(response);
+    free(response);
 }
 
 static HRESULT source_reader_get_stream_selection(const struct source_reader *reader, DWORD index, BOOL *selected)
@@ -1005,7 +1013,7 @@ static HRESULT source_reader_start_source(struct source_reader *reader)
         }
     }
 
-    position.u.hVal.QuadPart = 0;
+    position.hVal.QuadPart = 0;
     if (reader->source_state != SOURCE_STATE_STARTED || selection_changed)
     {
         position.vt = reader->source_state == SOURCE_STATE_STARTED ? VT_EMPTY : VT_I8;
@@ -1078,13 +1086,10 @@ static BOOL source_reader_get_read_result(struct source_reader *reader, struct m
 
 static HRESULT source_reader_get_next_selected_stream(struct source_reader *reader, unsigned int *stream_index)
 {
-    unsigned int i, start_idx, stop_idx, first_selected = ~0u, requests = ~0u;
+    unsigned int i, first_selected = ~0u, requests = ~0u;
     BOOL selected, stream_drained;
 
-    start_idx = (reader->last_read_index + 1) % reader->stream_count;
-    stop_idx = reader->last_read_index == ~0u ? reader->stream_count : reader->last_read_index;
-
-    for (i = start_idx; i < reader->stream_count && i != stop_idx; i = (i + 1) % (reader->stream_count + 1))
+    for (i = (reader->last_read_index + 1) % reader->stream_count; ; i = (i + 1) % reader->stream_count)
     {
         stream_drained = reader->streams[i].state == STREAM_STATE_EOS && !reader->streams[i].responses;
         selected = SUCCEEDED(source_reader_get_stream_selection(reader, i, &selected)) && selected;
@@ -1101,6 +1106,9 @@ static HRESULT source_reader_get_next_selected_stream(struct source_reader *read
                 *stream_index = i;
             }
         }
+
+        if (i == reader->last_read_index)
+            break;
     }
 
     /* If all selected streams reached EOS, use first selected. */
@@ -1163,8 +1171,8 @@ static void source_reader_flush_stream(struct source_reader *reader, DWORD strea
     struct media_stream *stream = &reader->streams[stream_index];
 
     source_reader_release_responses(reader, stream);
-    if (stream->decoder)
-        IMFTransform_ProcessMessage(stream->decoder, MFT_MESSAGE_COMMAND_FLUSH, 0);
+    if (stream->decoder.transform)
+        IMFTransform_ProcessMessage(stream->decoder.transform, MFT_MESSAGE_COMMAND_FLUSH, 0);
     stream->requests = 0;
 }
 
@@ -1380,15 +1388,16 @@ static ULONG WINAPI src_reader_Release(IMFSourceReader *iface)
                 IMFMediaStream_Release(stream->stream);
             if (stream->current)
                 IMFMediaType_Release(stream->current);
-            if (stream->decoder)
-                IMFTransform_Release(stream->decoder);
+            if (stream->decoder.transform)
+                IMFTransform_Release(stream->decoder.transform);
             if (stream->allocator)
                 IMFVideoSampleAllocatorEx_Release(stream->allocator);
         }
         source_reader_release_responses(reader, NULL);
-        heap_free(reader->streams);
+        free(reader->streams);
+        MFUnlockWorkQueue(reader->queue);
         DeleteCriticalSection(&reader->cs);
-        heap_free(reader);
+        free(reader);
     }
 
     return refcount;
@@ -1468,7 +1477,7 @@ static HRESULT WINAPI src_reader_SetStreamSelection(IMFSourceReader *iface, DWOR
     }
 
     if (selection_changed)
-        reader->last_read_index = ~0u;
+        reader->last_read_index = reader->stream_count - 1;
 
     LeaveCriticalSection(&reader->cs);
 
@@ -1620,27 +1629,20 @@ static HRESULT source_reader_set_compatible_media_type(struct source_reader *rea
     return type_set ? S_OK : S_FALSE;
 }
 
-static HRESULT source_reader_setup_sample_allocator(struct source_reader *reader, unsigned int index,
-        IMFMediaType *media_type)
+static HRESULT source_reader_setup_sample_allocator(struct source_reader *reader, unsigned int index)
 {
     struct media_stream *stream = &reader->streams[index];
     IMFVideoSampleAllocatorCallback *callback;
     GUID major = { 0 };
     HRESULT hr;
 
-    IMFMediaType_GetMajorType(media_type, &major);
+    IMFMediaType_GetMajorType(stream->current, &major);
     if (!IsEqualGUID(&major, &MFMediaType_Video))
         return S_OK;
 
     if (!(reader->flags & SOURCE_READER_HAS_DEVICE_MANAGER))
         return S_OK;
 
-    if (reader->flags & SOURCE_READER_DXGI_DEVICE_MANAGER)
-    {
-        FIXME("DXGI device manager is not supported.\n");
-        return S_OK;
-    }
-
     if (!stream->allocator)
     {
         if (FAILED(hr = MFCreateVideoSampleAllocatorEx(&IID_IMFVideoSampleAllocatorEx, (void **)&stream->allocator)))
@@ -1657,7 +1659,7 @@ static HRESULT source_reader_setup_sample_allocator(struct source_reader *reader
         return hr;
     }
 
-    if (FAILED(hr = IMFVideoSampleAllocatorEx_InitializeSampleAllocatorEx(stream->allocator, 2, 8, NULL, media_type)))
+    if (FAILED(hr = IMFVideoSampleAllocatorEx_InitializeSampleAllocatorEx(stream->allocator, 2, 8, NULL, stream->current)))
         WARN("Failed to initialize sample allocator, hr %#x.\n", hr);
 
     if (SUCCEEDED(IMFVideoSampleAllocatorEx_QueryInterface(stream->allocator, &IID_IMFVideoSampleAllocatorCallback, (void **)&callback)))
@@ -1674,8 +1676,10 @@ static HRESULT source_reader_configure_decoder(struct source_reader *reader, DWO
         IMFMediaType *input_type, IMFMediaType *output_type)
 {
     IMFMediaTypeHandler *type_handler;
+    unsigned int block_alignment = 0;
     IMFTransform *transform = NULL;
     IMFMediaType *type = NULL;
+    GUID major = { 0 };
     DWORD flags;
     HRESULT hr;
     int i = 0;
@@ -1712,12 +1716,15 @@ static HRESULT source_reader_configure_decoder(struct source_reader *reader, DWO
 
                     if (FAILED(hr = IMFMediaType_CopyAllItems(type, (IMFAttributes *)reader->streams[index].current)))
                         WARN("Failed to copy attributes, hr %#x.\n", hr);
+                    if (SUCCEEDED(IMFMediaType_GetMajorType(type, &major)) && IsEqualGUID(&major, &MFMediaType_Audio))
+                        IMFMediaType_GetUINT32(type, &MF_MT_AUDIO_BLOCK_ALIGNMENT, &block_alignment);
                     IMFMediaType_Release(type);
 
-                    if (reader->streams[index].decoder)
-                        IMFTransform_Release(reader->streams[index].decoder);
+                    if (reader->streams[index].decoder.transform)
+                        IMFTransform_Release(reader->streams[index].decoder.transform);
 
-                    reader->streams[index].decoder = transform;
+                    reader->streams[index].decoder.transform = transform;
+                    reader->streams[index].decoder.min_buffer_size = block_alignment;
 
                     return S_OK;
                 }
@@ -1823,7 +1830,7 @@ static HRESULT WINAPI src_reader_SetCurrentMediaType(IMFSourceReader *iface, DWO
     if (hr == S_FALSE)
         hr = source_reader_create_decoder_for_stream(reader, index, type);
     if (SUCCEEDED(hr))
-        hr = source_reader_setup_sample_allocator(reader, index, type);
+        hr = source_reader_setup_sample_allocator(reader, index);
 
     LeaveCriticalSection(&reader->cs);
 
@@ -1866,8 +1873,7 @@ static HRESULT WINAPI src_reader_SetCurrentPosition(IMFSourceReader *iface, REFG
                 command->u.seek.format = *format;
                 PropVariantCopy(&command->u.seek.position, position);
 
-                hr = MFPutWorkItem(MFASYNC_CALLBACK_QUEUE_MULTITHREADED, &reader->async_commands_callback,
-                        &command->IUnknown_iface);
+                hr = MFPutWorkItem(reader->queue, &reader->async_commands_callback, &command->IUnknown_iface);
                 IUnknown_Release(&command->IUnknown_iface);
             }
         }
@@ -1969,7 +1975,7 @@ static HRESULT source_reader_read_sample_async(struct source_reader *reader, uns
             command->u.read.stream_index = index;
             command->u.read.flags = flags;
 
-            hr = MFPutWorkItem(MFASYNC_CALLBACK_QUEUE_STANDARD, &reader->async_commands_callback, &command->IUnknown_iface);
+            hr = MFPutWorkItem(reader->queue, &reader->async_commands_callback, &command->IUnknown_iface);
             IUnknown_Release(&command->IUnknown_iface);
         }
     }
@@ -2033,7 +2039,7 @@ static HRESULT source_reader_flush_async(struct source_reader *reader, unsigned
 
     command->u.flush.stream_index = stream_index;
 
-    hr = MFPutWorkItem(MFASYNC_CALLBACK_QUEUE_STANDARD, &reader->async_commands_callback, &command->IUnknown_iface);
+    hr = MFPutWorkItem(reader->queue, &reader->async_commands_callback, &command->IUnknown_iface);
     IUnknown_Release(&command->IUnknown_iface);
 
     return hr;
@@ -2084,7 +2090,7 @@ static HRESULT WINAPI src_reader_GetServiceForStream(IMFSourceReader *iface, DWO
                 hr = MF_E_INVALIDSTREAMNUMBER;
             else
             {
-                obj = (IUnknown *)reader->streams[index].decoder;
+                obj = (IUnknown *)reader->streams[index].decoder.transform;
                 if (!obj) hr = E_NOINTERFACE;
             }
             break;
@@ -2141,7 +2147,7 @@ static HRESULT WINAPI src_reader_GetPresentationAttribute(IMFSourceReader *iface
                     return hr;
 
                 value->vt = VT_UI4;
-                value->u.ulVal = flags;
+                value->ulVal = flags;
                 return S_OK;
             }
             else
@@ -2258,7 +2264,7 @@ static HRESULT WINAPI stream_sample_allocator_cb_NotifyRelease(IMFVideoSampleAll
     if (SUCCEEDED(source_reader_create_async_op(SOURCE_READER_ASYNC_SA_READY, &command)))
     {
         command->u.sa.stream_index = stream->index;
-        MFPutWorkItem(MFASYNC_CALLBACK_QUEUE_STANDARD, &stream->reader->async_commands_callback, &command->IUnknown_iface);
+        MFPutWorkItem(stream->reader->queue, &stream->reader->async_commands_callback, &command->IUnknown_iface);
         IUnknown_Release(&command->IUnknown_iface);
     }
 
@@ -2280,7 +2286,7 @@ static HRESULT create_source_reader_from_source(IMFMediaSource *source, IMFAttri
     unsigned int i;
     HRESULT hr;
 
-    object = heap_alloc_zero(sizeof(*object));
+    object = calloc(1, sizeof(*object));
     if (!object)
         return E_OUTOFMEMORY;
 
@@ -2304,7 +2310,7 @@ static HRESULT create_source_reader_from_source(IMFMediaSource *source, IMFAttri
     if (FAILED(hr = IMFPresentationDescriptor_GetStreamDescriptorCount(object->descriptor, &object->stream_count)))
         goto failed;
 
-    if (!(object->streams = heap_alloc_zero(object->stream_count * sizeof(*object->streams))))
+    if (!(object->streams = calloc(object->stream_count, sizeof(*object->streams))))
     {
         hr = E_OUTOFMEMORY;
         goto failed;
@@ -2353,7 +2359,7 @@ static HRESULT create_source_reader_from_source(IMFMediaSource *source, IMFAttri
     /* At least one major type has to be set. */
     object->first_audio_stream_index = reader_get_first_stream_index(object->descriptor, &MFMediaType_Audio);
     object->first_video_stream_index = reader_get_first_stream_index(object->descriptor, &MFMediaType_Video);
-    object->last_read_index = ~0u;
+    object->last_read_index = object->stream_count - 1;
 
     if (object->first_audio_stream_index == MF_SOURCE_READER_INVALID_STREAM_INDEX &&
             object->first_video_stream_index == MF_SOURCE_READER_INVALID_STREAM_INDEX)
@@ -2399,7 +2405,11 @@ static HRESULT create_source_reader_from_source(IMFMediaSource *source, IMFAttri
         }
     }
 
-    hr = IMFSourceReader_QueryInterface(&object->IMFSourceReader_iface, riid, out);
+    if (FAILED(hr = MFLockSharedWorkQueue(L"", 0, NULL, &object->queue)))
+        WARN("Failed to acquired shared queue, hr %#x.\n", hr);
+
+    if (SUCCEEDED(hr))
+        hr = IMFSourceReader_QueryInterface(&object->IMFSourceReader_iface, riid, out);
 
 failed:
     IMFSourceReader_Release(&object->IMFSourceReader_iface);
