@@ -51,9 +51,6 @@
 #ifdef HAVE_SYS_RESOURCE_H
 # include <sys/resource.h>
 #endif
-#ifdef HAVE_SYS_UTSNAME_H
-#include <sys/utsname.h>
-#endif
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif
@@ -94,8 +91,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(module);
 
-#define IMAGE_DLLCHARACTERISTICS_PREFER_NATIVE 0x0010 /* Wine extension */
-
 void     (WINAPI *pDbgUiRemoteBreakin)( void *arg ) = NULL;
 NTSTATUS (WINAPI *pKiRaiseUserExceptionDispatcher)(void) = NULL;
 void     (WINAPI *pKiUserApcDispatcher)(CONTEXT*,ULONG_PTR,ULONG_PTR,ULONG_PTR,PNTAPCFUNC) = NULL;
@@ -116,8 +111,6 @@ static const BOOL use_preloader = TRUE;
 static const BOOL use_preloader = FALSE;
 #endif
 
-static const BOOL is_win64 = (sizeof(void *) > sizeof(int));
-
 static char *argv0;
 static const char *bin_dir;
 static const char *dll_dir;
@@ -129,43 +122,10 @@ const char *build_dir = NULL;
 const char *config_dir = NULL;
 const char **dll_paths = NULL;
 const char *user_name = NULL;
+SECTION_IMAGE_INFORMATION main_image_info = { NULL };
 static HMODULE ntdll_module;
 static const IMAGE_EXPORT_DIRECTORY *ntdll_exports;
 
-struct file_id
-{
-    dev_t dev;
-    ino_t ino;
-};
-
-struct builtin_module
-{
-    struct list    entry;
-    struct file_id id;
-    void          *handle;
-    void          *module;
-    void          *unix_handle;
-};
-
-static struct list builtin_modules = LIST_INIT( builtin_modules );
-
-static NTSTATUS add_builtin_module( void *module, void *handle, const struct stat *st )
-{
-    struct builtin_module *builtin;
-    if (!(builtin = malloc( sizeof(*builtin) ))) return STATUS_NO_MEMORY;
-    builtin->handle = handle;
-    builtin->module = module;
-    builtin->unix_handle = NULL;
-    if (st)
-    {
-        builtin->id.dev = st->st_dev;
-        builtin->id.ino = st->st_ino;
-    }
-    else memset( &builtin->id, 0, sizeof(builtin->id) );
-    list_add_tail( &builtin_modules, &builtin->entry );
-    return STATUS_SUCCESS;
-}
-
 /* adjust an array of pointers to make them into RVAs */
 static inline void fixup_rva_ptrs( void *array, BYTE *base, unsigned int count )
 {
@@ -373,48 +333,6 @@ static void init_paths( char *argv[] )
 }
 
 
-/*********************************************************************
- *                  wine_get_version
- */
-const char * CDECL wine_get_version(void)
-{
-    return PACKAGE_VERSION;
-}
-
-
-/*********************************************************************
- *                  wine_get_build_id
- */
-const char * CDECL wine_get_build_id(void)
-{
-    extern const char wine_build[];
-    return wine_build;
-}
-
-
-/*********************************************************************
- *                  wine_get_host_version
- */
-void CDECL wine_get_host_version( const char **sysname, const char **release )
-{
-#ifdef HAVE_SYS_UTSNAME_H
-    static struct utsname buf;
-    static BOOL init_done;
-
-    if (!init_done)
-    {
-        uname( &buf );
-        init_done = TRUE;
-    }
-    if (sysname) *sysname = buf.sysname;
-    if (release) *release = buf.release;
-#else
-    if (sysname) *sysname = "";
-    if (release) *release = "";
-#endif
-}
-
-
 static void preloader_exec( char **argv )
 {
     if (use_preloader)
@@ -445,13 +363,13 @@ static void preloader_exec( char **argv )
     execv( argv[1], argv + 1 );
 }
 
-static NTSTATUS loader_exec( const char *loader, char **argv, client_cpu_t cpu )
+static NTSTATUS loader_exec( const char *loader, char **argv, WORD machine )
 {
     char *p, *path;
 
     if (build_dir)
     {
-        argv[1] = build_path( build_dir, (cpu == CPU_x86_64) ? "loader/wine64" : "loader/wine" );
+        argv[1] = build_path( build_dir, (machine == IMAGE_FILE_MACHINE_AMD64) ? "loader/wine64" : "loader/wine" );
         preloader_exec( argv );
         return STATUS_INVALID_IMAGE_FORMAT;
     }
@@ -486,13 +404,20 @@ static NTSTATUS loader_exec( const char *loader, char **argv, client_cpu_t cpu )
  */
 NTSTATUS exec_wineloader( char **argv, int socketfd, const pe_image_info_t *pe_info )
 {
-    int is_child_64bit = (pe_info->cpu == CPU_x86_64 || pe_info->cpu == CPU_ARM64);
+    WORD machine = pe_info->machine;
+    int is_child_64bit = (machine == IMAGE_FILE_MACHINE_AMD64 || machine == IMAGE_FILE_MACHINE_ARM64);
     ULONGLONG res_start = pe_info->base;
     ULONGLONG res_end = pe_info->base + pe_info->map_size;
     const char *loader = argv0;
     const char *loader_env = getenv( "WINELOADER" );
     char preloader_reserve[64], socket_env[64];
 
+    if (!is_child_64bit && (is_win64 || is_wow64) && (pe_info->image_flags & IMAGE_FLAGS_ComPlusNativeReady))
+    {
+        is_child_64bit = TRUE;
+        machine = IMAGE_FILE_MACHINE_AMD64;
+    }
+
     if (!is_win64 ^ !is_child_64bit)
     {
         /* remap WINELOADER to the alternate 32/64-bit version if necessary */
@@ -528,7 +453,7 @@ NTSTATUS exec_wineloader( char **argv, int socketfd, const pe_image_info_t *pe_i
     putenv( preloader_reserve );
     putenv( socket_env );
 
-    return loader_exec( loader, argv, pe_info->cpu );
+    return loader_exec( loader, argv, machine );
 }
 
 
@@ -647,6 +572,7 @@ static NTSTATUS map_so_dll( const IMAGE_NT_HEADERS *nt_descr, HMODULE module )
     dos->e_ss       = 0x0000;
     dos->e_sp       = 0x00b8;
     dos->e_lfanew   = sizeof(*dos) + sizeof(builtin_signature);
+    memcpy( dos + 1, builtin_signature, sizeof(builtin_signature) );
 
     *nt = *nt_descr;
 
@@ -1068,8 +994,8 @@ static void fill_builtin_image_info( void *module, pe_image_info_t *info )
     info->header_size     = nt->OptionalHeader.SizeOfHeaders;
     info->file_size       = nt->OptionalHeader.SizeOfImage;
     info->checksum        = nt->OptionalHeader.CheckSum;
-    info->cpu             = client_cpu;
-    info->__pad           = 0;
+    info->dbg_offset      = 0;
+    info->dbg_size        = 0;
 }
 
 
@@ -1079,7 +1005,6 @@ static void fill_builtin_image_info( void *module, pe_image_info_t *info )
 static NTSTATUS dlopen_dll( const char *so_name, UNICODE_STRING *nt_name, void **ret_module,
                             pe_image_info_t *image_info, BOOL prefer_native )
 {
-    struct builtin_module *builtin;
     void *module, *handle;
     const IMAGE_NT_HEADERS *nt;
 
@@ -1095,14 +1020,12 @@ static NTSTATUS dlopen_dll( const char *so_name, UNICODE_STRING *nt_name, void *
         if (!callback_module) return STATUS_NO_MEMORY;
         WARN( "got old-style builtin library %s, constructors won't work\n", debugstr_a(so_name) );
         module = callback_module;
-        LIST_FOR_EACH_ENTRY( builtin, &builtin_modules, struct builtin_module, entry )
-            if (builtin->module == module) goto already_loaded;
+        if (get_builtin_so_handle( module )) goto already_loaded;
     }
     else if ((nt = dlsym( handle, "__wine_spec_nt_header" )))
     {
         module = (HMODULE)((nt->OptionalHeader.ImageBase + 0xffff) & ~0xffff);
-        LIST_FOR_EACH_ENTRY( builtin, &builtin_modules, struct builtin_module, entry )
-            if (builtin->module == module) goto already_loaded;
+        if (get_builtin_so_handle( module )) goto already_loaded;
         if (map_so_dll( nt, module ))
         {
             dlclose( handle );
@@ -1112,8 +1035,6 @@ static NTSTATUS dlopen_dll( const char *so_name, UNICODE_STRING *nt_name, void *
     else  /* already loaded .so */
     {
         WARN( "%s already loaded?\n", debugstr_a(so_name));
-        LIST_FOR_EACH_ENTRY( builtin, &builtin_modules, struct builtin_module, entry )
-            if (builtin->handle == handle) goto already_loaded;
         return STATUS_INVALID_IMAGE_FORMAT;
     }
 
@@ -1125,70 +1046,55 @@ static NTSTATUS dlopen_dll( const char *so_name, UNICODE_STRING *nt_name, void *
         return STATUS_IMAGE_ALREADY_LOADED;
     }
 
-    if (add_builtin_module( module, handle, NULL ))
+    if (virtual_create_builtin_view( module, nt_name, image_info, handle ))
     {
         dlclose( handle );
         return STATUS_NO_MEMORY;
     }
-    virtual_create_builtin_view( module, nt_name, image_info );
     *ret_module = module;
     return STATUS_SUCCESS;
 
 already_loaded:
-    fill_builtin_image_info( builtin->module, image_info );
-    *ret_module = builtin->module;
+    fill_builtin_image_info( module, image_info );
+    *ret_module = module;
     dlclose( handle );
     return STATUS_SUCCESS;
 }
 
 
 /***********************************************************************
- *           dlopen_unix_dll
+ *           init_unix_lib
  */
-static NTSTATUS dlopen_unix_dll( void *module, const char *name, void **unix_entry )
+static NTSTATUS CDECL init_unix_lib( void *module, DWORD reason, const void *ptr_in, void *ptr_out )
 {
-    struct builtin_module *builtin;
-    void *unix_module, *handle, *entry;
+    NTSTATUS (CDECL *init_func)( HMODULE, DWORD, const void *, void * );
     const IMAGE_NT_HEADERS *nt;
-    NTSTATUS status = STATUS_INVALID_IMAGE_FORMAT;
+    const char *name;
+    void *handle, *entry, *unix_module;
+    NTSTATUS status;
 
-    handle = dlopen( name, RTLD_NOW );
-    if (!handle) return STATUS_DLL_NOT_FOUND;
-    if (!(nt = dlsym( handle, "__wine_spec_nt_header" ))) goto done;
-    if (!(entry = dlsym( handle, "__wine_init_unix_lib" ))) goto done;
+    if ((status = get_builtin_unix_info( module, &name, &handle, &entry ))) return status;
 
-    unix_module = (HMODULE)((nt->OptionalHeader.ImageBase + 0xffff) & ~0xffff);
-    LIST_FOR_EACH_ENTRY( builtin, &builtin_modules, struct builtin_module, entry )
+    if (!entry)
     {
-        if (builtin->module == module)
+        if (!name) return STATUS_DLL_NOT_FOUND;
+        if (!(handle = dlopen( name, RTLD_NOW ))) return STATUS_DLL_NOT_FOUND;
+
+        if (!(nt = dlsym( handle, "__wine_spec_nt_header" )) ||
+            !(entry = dlsym( handle, "__wine_init_unix_lib" )))
         {
-            if (builtin->unix_handle == handle)  /* already loaded */
-            {
-                *unix_entry = entry;
-                status = STATUS_SUCCESS;
-                goto done;
-            }
-            if (builtin->unix_handle)
-            {
-                ERR( "module %p already has a Unix module that's not %s\n", module, debugstr_a(name) );
-                goto done;
-            }
-            if ((status = map_so_dll( nt, unix_module ))) goto done;
-            if ((status = fixup_ntdll_imports( name, unix_module ))) goto done;
-            builtin->unix_handle = handle;
-            *unix_entry = entry;
-            return STATUS_SUCCESS;
-        }
-        else if (builtin->unix_handle == handle)
-        {
-            ERR( "%s already loaded for module %p\n", debugstr_a(name), module );
-            goto done;
+            dlclose( handle );
+            set_builtin_unix_info( module, NULL, NULL, NULL );
+            return STATUS_INVALID_IMAGE_FORMAT;
         }
+        TRACE( "loaded %s for %p\n", debugstr_a(name), module );
+        unix_module = (void *)((nt->OptionalHeader.ImageBase + 0xffff) & ~0xffff);
+        map_so_dll( nt, unix_module );
+        fixup_ntdll_imports( name, unix_module );
+        set_builtin_unix_info( module, NULL, handle, entry );
     }
-    ERR( "builtin module not found for %s\n", debugstr_a(name) );
-done:
-    dlclose( handle );
-    return status;
+    init_func = entry;
+    return init_func( module, reason, ptr_in, ptr_out );
 }
 
 
@@ -1203,6 +1109,7 @@ static NTSTATUS CDECL load_so_dll( UNICODE_STRING *nt_name, void **module )
     NTSTATUS status;
     DWORD len;
 
+    if (get_load_order( nt_name ) == LO_DISABLED) return STATUS_DLL_NOT_FOUND;
     if (nt_to_unix_file_name( nt_name, &unix_name, NULL, FILE_OPEN )) return STATUS_DLL_NOT_FOUND;
 
     /* remove .so extension from Windows name */
@@ -1215,23 +1122,6 @@ static NTSTATUS CDECL load_so_dll( UNICODE_STRING *nt_name, void **module )
 }
 
 
-/* check a PE library architecture */
-static BOOL is_valid_binary( const SECTION_IMAGE_INFORMATION *info )
-{
-#ifdef __i386__
-    return info->Machine == IMAGE_FILE_MACHINE_I386;
-#elif defined(__arm__)
-    return info->Machine == IMAGE_FILE_MACHINE_ARM ||
-           info->Machine == IMAGE_FILE_MACHINE_THUMB ||
-           info->Machine == IMAGE_FILE_MACHINE_ARMNT;
-#elif defined(__x86_64__)
-    /* we don't support 32-bit IL-only builtins yet */
-    return info->Machine == IMAGE_FILE_MACHINE_AMD64;
-#elif defined(__aarch64__)
-    return info->Machine == IMAGE_FILE_MACHINE_ARM64;
-#endif
-}
-
 /* check if the library is the correct architecture */
 /* only returns false for a valid library of the wrong arch */
 static int check_library_arch( int fd )
@@ -1279,10 +1169,8 @@ static inline char *prepend( char *buffer, const char *str, size_t len )
  *
  * Open a file for a new dll. Helper for open_builtin_file.
  */
-static NTSTATUS open_dll_file( const char *name, OBJECT_ATTRIBUTES *attr, HANDLE *mapping, void **module,
-                               SECTION_IMAGE_INFORMATION *image_info, struct stat *st, BOOL prefer_native )
+static NTSTATUS open_dll_file( const char *name, OBJECT_ATTRIBUTES *attr, HANDLE *mapping )
 {
-    struct builtin_module *builtin;
     LARGE_INTEGER size;
     NTSTATUS status;
     HANDLE handle;
@@ -1294,56 +1182,18 @@ static NTSTATUS open_dll_file( const char *name, OBJECT_ATTRIBUTES *attr, HANDLE
         if (status != STATUS_OBJECT_PATH_NOT_FOUND && status != STATUS_OBJECT_NAME_NOT_FOUND)
         {
             /* if the file exists but failed to open, report the error */
-            if (!stat( name, st )) return status;
+            struct stat st;
+            if (!stat( name, &st )) return status;
         }
         /* otherwise continue searching */
         return STATUS_DLL_NOT_FOUND;
     }
 
-    if (!stat( name, st ))
-    {
-        LIST_FOR_EACH_ENTRY( builtin, &builtin_modules, struct builtin_module, entry )
-        {
-            if (builtin->id.dev == st->st_dev && builtin->id.ino == st->st_ino)
-            {
-                TRACE( "%s is the same file as existing module %p\n", debugstr_a(name),
-                       builtin->module );
-                NtClose( handle );
-                *module = builtin->module;
-                return STATUS_SUCCESS;
-            }
-        }
-    }
-    else memset( st, 0, sizeof(*st) );
-
-    *module = NULL;
     size.QuadPart = 0;
     status = NtCreateSection( mapping, STANDARD_RIGHTS_REQUIRED | SECTION_QUERY |
                               SECTION_MAP_READ | SECTION_MAP_EXECUTE,
                               NULL, &size, PAGE_EXECUTE_READ, SEC_IMAGE, handle );
     NtClose( handle );
-    if (status) return status;
-
-    NtQuerySection( *mapping, SectionImageInformation, image_info, sizeof(*image_info), NULL );
-    /* ignore non-builtins */
-    if (!(image_info->u.s.WineBuiltin))
-    {
-        WARN( "%s found in WINEDLLPATH but not a builtin, ignoring\n", debugstr_a(name) );
-        NtClose( *mapping );
-        return STATUS_DLL_NOT_FOUND;
-    }
-    if (!is_valid_binary( image_info ))
-    {
-        TRACE( "%s is for arch %x, continuing search\n", debugstr_a(name), image_info->Machine );
-        NtClose( *mapping );
-        return STATUS_IMAGE_MACHINE_TYPE_MISMATCH;
-    }
-    if (prefer_native && (image_info->DllCharacteristics & IMAGE_DLLCHARACTERISTICS_PREFER_NATIVE))
-    {
-        TRACE( "%s has prefer-native flag, ignoring builtin\n", debugstr_a(name) );
-        NtClose( *mapping );
-        return STATUS_IMAGE_ALREADY_LOADED;
-    }
     return status;
 }
 
@@ -1351,14 +1201,20 @@ static NTSTATUS open_dll_file( const char *name, OBJECT_ATTRIBUTES *attr, HANDLE
 /***********************************************************************
  *           open_builtin_file
  */
-static NTSTATUS open_builtin_file( char *name, OBJECT_ATTRIBUTES *attr, HANDLE *mapping, void **module,
-                                   SECTION_IMAGE_INFORMATION *image_info, struct stat *st,
-                                   BOOL prefer_native )
+static NTSTATUS open_builtin_file( char *name, OBJECT_ATTRIBUTES *attr, void **module, SIZE_T *size,
+                                   SECTION_IMAGE_INFORMATION *image_info, WORD machine, BOOL prefer_native )
 {
     NTSTATUS status;
+    HANDLE mapping;
     int fd;
 
-    status = open_dll_file( name, attr, mapping, module, image_info, st, prefer_native );
+    *module = NULL;
+    status = open_dll_file( name, attr, &mapping );
+    if (!status)
+    {
+        status = virtual_map_builtin_module( mapping, module, size, image_info, machine, prefer_native );
+        NtClose( mapping );
+    }
     if (status != STATUS_DLL_NOT_FOUND) return status;
 
     /* try .so file */
@@ -1386,32 +1242,10 @@ static NTSTATUS open_builtin_file( char *name, OBJECT_ATTRIBUTES *attr, HANDLE *
 
 
 /***********************************************************************
- *           map_builtin_module
+ *           find_builtin_dll
  */
-static NTSTATUS map_builtin_module( HANDLE mapping, void **module, struct stat *st )
-{
-    NTSTATUS status;
-    SIZE_T len = 0;
-
-    *module = NULL;
-    status = NtMapViewOfSection( mapping, NtCurrentProcess(), module, 0, 0, NULL, &len,
-                                 ViewShare, 0, PAGE_EXECUTE_READ );
-    if (status == STATUS_IMAGE_NOT_AT_BASE) status = STATUS_SUCCESS;
-
-    if (!status)
-    {
-        status = add_builtin_module( *module, NULL, st );
-        if (status) NtUnmapViewOfSection( NtCurrentProcess(), *module );
-    }
-    return status;
-}
-
-
-/***********************************************************************
- *           load_builtin_dll
- */
-static NTSTATUS CDECL load_builtin_dll( UNICODE_STRING *nt_name, void **module, void **unix_entry,
-                                        SECTION_IMAGE_INFORMATION *image_info, BOOL prefer_native )
+static NTSTATUS find_builtin_dll( UNICODE_STRING *nt_name, void **module, SIZE_T *size_ptr,
+                                  SECTION_IMAGE_INFORMATION *image_info, WORD machine, BOOL prefer_native )
 {
     unsigned int i, pos, namepos, namelen, maxlen = 0;
     unsigned int len = nt_name->Length / sizeof(WCHAR);
@@ -1419,8 +1253,6 @@ static NTSTATUS CDECL load_builtin_dll( UNICODE_STRING *nt_name, void **module,
     OBJECT_ATTRIBUTES attr;
     NTSTATUS status = STATUS_DLL_NOT_FOUND;
     BOOL found_image = FALSE;
-    HANDLE mapping;
-    struct stat st;
 
     for (i = namepos = 0; i < len; i++)
         if (nt_name->Buffer[i] == '/' || nt_name->Buffer[i] == '\\') namepos = i + 1;
@@ -1454,7 +1286,7 @@ static NTSTATUS CDECL load_builtin_dll( UNICODE_STRING *nt_name, void **module,
         ptr = prepend( ptr, ptr, namelen );
         ptr = prepend( ptr, "/dlls", sizeof("/dlls") - 1 );
         ptr = prepend( ptr, build_dir, strlen(build_dir) );
-        status = open_builtin_file( ptr, &attr, &mapping, module, image_info, &st, prefer_native );
+        status = open_builtin_file( ptr, &attr, module, size_ptr, image_info, machine, prefer_native );
         if (status != STATUS_DLL_NOT_FOUND) goto done;
 
         /* now as a program */
@@ -1465,7 +1297,7 @@ static NTSTATUS CDECL load_builtin_dll( UNICODE_STRING *nt_name, void **module,
         ptr = prepend( ptr, ptr, namelen );
         ptr = prepend( ptr, "/programs", sizeof("/programs") - 1 );
         ptr = prepend( ptr, build_dir, strlen(build_dir) );
-        status = open_builtin_file( ptr, &attr, &mapping, module, image_info, &st, prefer_native );
+        status = open_builtin_file( ptr, &attr, module, size_ptr, image_info, machine, prefer_native );
         if (status != STATUS_DLL_NOT_FOUND) goto done;
     }
 
@@ -1473,24 +1305,18 @@ static NTSTATUS CDECL load_builtin_dll( UNICODE_STRING *nt_name, void **module,
     {
         file[pos + len + 1] = 0;
         ptr = prepend( file + pos, dll_paths[i], strlen(dll_paths[i]) );
-        status = open_builtin_file( ptr, &attr, &mapping, module, image_info, &st, prefer_native );
+        status = open_builtin_file( ptr, &attr, module, size_ptr, image_info, machine, prefer_native );
         if (status == STATUS_IMAGE_MACHINE_TYPE_MISMATCH) found_image = TRUE;
         else if (status != STATUS_DLL_NOT_FOUND) goto done;
     }
 
     if (found_image) status = STATUS_IMAGE_MACHINE_TYPE_MISMATCH;
     WARN( "cannot find builtin library for %s\n", debugstr_us(nt_name) );
-
 done:
-    if (!status && !*module)
-    {
-        status = map_builtin_module( mapping, module, &st );
-        NtClose( mapping );
-    }
-    if (!status && ext)
+    if (status >= 0 && ext)
     {
         strcpy( ext, ".so" );
-        dlopen_unix_dll( *module, ptr, unix_entry );
+        set_builtin_unix_info( *module, ptr, NULL, NULL );
     }
     free( file );
     return status;
@@ -1498,22 +1324,222 @@ done:
 
 
 /***********************************************************************
- *           unload_builtin_dll
+ *           load_builtin
+ *
+ * Load the builtin dll if specified by load order configuration.
+ * Return STATUS_IMAGE_ALREADY_LOADED if we should keep the native one that we have found.
  */
-static NTSTATUS CDECL unload_builtin_dll( void *module )
+NTSTATUS load_builtin( const pe_image_info_t *image_info, WCHAR *filename,
+                       void **module, SIZE_T *size )
 {
-    struct builtin_module *builtin;
+    WORD machine = image_info->machine;  /* request same machine as the native one */
+    NTSTATUS status;
+    UNICODE_STRING nt_name;
+    SECTION_IMAGE_INFORMATION info;
+    enum loadorder loadorder;
 
-    LIST_FOR_EACH_ENTRY( builtin, &builtin_modules, struct builtin_module, entry )
+    /* remove .fake extension if present */
+    if (image_info->image_flags & IMAGE_FLAGS_WineFakeDll)
     {
-        if (builtin->module != module) continue;
-        list_remove( &builtin->entry );
-        if (builtin->handle) dlclose( builtin->handle );
-        if (builtin->unix_handle) dlclose( builtin->unix_handle );
-        free( builtin );
-        return STATUS_SUCCESS;
+        static const WCHAR fakeW[] = {'.','f','a','k','e',0};
+        WCHAR *ext = wcsrchr( filename, '.' );
+
+        TRACE( "%s is a fake Wine dll\n", debugstr_w(filename) );
+        if (ext && !wcsicmp( ext, fakeW )) *ext = 0;
     }
-    return STATUS_INVALID_PARAMETER;
+
+    init_unicode_string( &nt_name, filename );
+    loadorder = get_load_order( &nt_name );
+
+    if (loadorder == LO_DISABLED) return STATUS_DLL_NOT_FOUND;
+
+    if (image_info->image_flags & IMAGE_FLAGS_WineBuiltin)
+    {
+        if (loadorder == LO_NATIVE) return STATUS_DLL_NOT_FOUND;
+        loadorder = LO_BUILTIN_NATIVE;  /* load builtin, then fallback to the file we found */
+    }
+    else if (image_info->image_flags & IMAGE_FLAGS_WineFakeDll)
+    {
+        if (loadorder == LO_NATIVE) return STATUS_DLL_NOT_FOUND;
+        loadorder = LO_BUILTIN;  /* builtin with no fallback since mapping a fake dll is not useful */
+    }
+
+    switch (loadorder)
+    {
+    case LO_NATIVE:
+    case LO_NATIVE_BUILTIN:
+        return STATUS_IMAGE_ALREADY_LOADED;
+    case LO_BUILTIN:
+        return find_builtin_dll( &nt_name, module, size, &info, machine, FALSE );
+    default:
+        status = find_builtin_dll( &nt_name, module, size, &info, machine, (loadorder == LO_DEFAULT) );
+        if (status == STATUS_DLL_NOT_FOUND || status == STATUS_IMAGE_MACHINE_TYPE_MISMATCH)
+            return STATUS_IMAGE_ALREADY_LOADED;
+        return status;
+    }
+}
+
+
+/***************************************************************************
+ *	is_builtin_path
+ */
+BOOL is_builtin_path( const UNICODE_STRING *path, WORD *machine )
+{
+    static const WCHAR wow64W[] = {'\\','?','?','\\','c',':','\\','w','i','n','d','o','w','s','\\',
+                                   's','y','s','w','o','w','6','4'};
+    unsigned int len;
+
+    /* only fake builtin existence during prefix bootstrap */
+    if (!is_prefix_bootstrap) return FALSE;
+
+    *machine = current_machine;
+    if (path->Length > wcslen(system_dir) * sizeof(WCHAR) &&
+        !wcsnicmp( path->Buffer, system_dir, wcslen(system_dir) ))
+    {
+        if (NtCurrentTeb64() && NtCurrentTeb64()->TlsSlots[WOW64_TLS_FILESYSREDIR])
+            *machine = IMAGE_FILE_MACHINE_AMD64;
+        goto found;
+    }
+    if ((is_win64 || is_wow64) && path->Length > sizeof(wow64W) &&
+        !wcsnicmp( path->Buffer, wow64W, ARRAY_SIZE(wow64W) ))
+    {
+        *machine = IMAGE_FILE_MACHINE_I386;
+        goto found;
+    }
+    return FALSE;
+
+found:
+    /* check that there are no other path components */
+    len = wcslen(system_dir);
+    while (len < path->Length / sizeof(WCHAR) && path->Buffer[len] == '\\') len++;
+    while (len < path->Length / sizeof(WCHAR) && path->Buffer[len] != '\\') len++;
+    return (len == path->Length / sizeof(WCHAR));
+}
+
+
+/***********************************************************************
+ *           open_main_image
+ */
+static NTSTATUS open_main_image( WCHAR *image, void **module, SECTION_IMAGE_INFORMATION *info )
+{
+    static const WCHAR soW[] = {'.','s','o',0};
+    UNICODE_STRING nt_name;
+    OBJECT_ATTRIBUTES attr;
+    pe_image_info_t pe_info;
+    SIZE_T size = 0;
+    char *unix_name;
+    NTSTATUS status;
+    HANDLE mapping;
+    WCHAR *p;
+
+    init_unicode_string( &nt_name, image );
+    InitializeObjectAttributes( &attr, &nt_name, OBJ_CASE_INSENSITIVE, 0, NULL );
+    if (nt_to_unix_file_name( &nt_name, &unix_name, NULL, FILE_OPEN )) return STATUS_DLL_NOT_FOUND;
+
+    status = open_dll_file( unix_name, &attr, &mapping );
+    if (!status)
+    {
+        *module = NULL;
+        status = NtMapViewOfSection( mapping, NtCurrentProcess(), module, 0, 0, NULL, &size,
+                                     ViewShare, 0, PAGE_EXECUTE_READ );
+        if (!status) NtQuerySection( mapping, SectionImageInformation, info, sizeof(*info), NULL );
+        NtClose( mapping );
+    }
+    else if (status == STATUS_INVALID_IMAGE_NOT_MZ)
+    {
+        /* remove .so extension from Windows name */
+        p = image + wcslen(image);
+        if (p - image > 3 && !wcsicmp( p - 3, soW ))
+        {
+            p[-3] = 0;
+            nt_name.Length -= 3 * sizeof(WCHAR);
+        }
+        status = dlopen_dll( unix_name, &nt_name, module, &pe_info, FALSE );
+        if (!status) virtual_fill_image_information( &pe_info, info );
+    }
+    free( unix_name );
+    return status;
+}
+
+
+/***********************************************************************
+ *           load_main_exe
+ */
+NTSTATUS load_main_exe( const WCHAR *dos_name, const char *unix_name, const WCHAR *curdir,
+                        WCHAR **image, void **module )
+{
+    UNICODE_STRING nt_name;
+    WCHAR *tmp = NULL;
+    BOOL contains_path;
+    NTSTATUS status;
+    SIZE_T size;
+    struct stat st;
+    WORD machine;
+
+    /* special case for Unix file name */
+    if (unix_name && unix_name[0] == '/' && !stat( unix_name, &st ))
+    {
+        if ((status = unix_to_nt_file_name( unix_name, image ))) goto failed;
+        status = open_main_image( *image, module, &main_image_info );
+        if (status != STATUS_DLL_NOT_FOUND) return status;
+        free( *image );
+    }
+
+    if (!dos_name)
+    {
+        dos_name = tmp = malloc( (strlen(unix_name) + 1) * sizeof(WCHAR) );
+        ntdll_umbstowcs( unix_name, strlen(unix_name) + 1, tmp, strlen(unix_name) + 1 );
+    }
+    contains_path = (wcschr( dos_name, '/' ) ||
+                     wcschr( dos_name, '\\' ) ||
+                     (dos_name[0] && dos_name[1] == ':'));
+
+    if ((status = get_full_path( dos_name, curdir, image ))) goto failed;
+    free( tmp );
+
+    status = open_main_image( *image, module, &main_image_info );
+    if (status != STATUS_DLL_NOT_FOUND) return status;
+
+    /* if path is in system dir, we can load the builtin even if the file itself doesn't exist */
+    init_unicode_string( &nt_name, *image );
+    if (is_builtin_path( &nt_name, &machine ))
+    {
+        status = find_builtin_dll( &nt_name, module, &size, &main_image_info, machine, FALSE );
+        if (status != STATUS_DLL_NOT_FOUND) return status;
+    }
+    if (!contains_path) return STATUS_DLL_NOT_FOUND;
+
+failed:
+    MESSAGE( "wine: failed to open %s: %x\n",
+             unix_name ? debugstr_a(unix_name) : debugstr_w(dos_name), status );
+    NtTerminateProcess( GetCurrentProcess(), status );
+    return status;  /* unreached */
+}
+
+
+/***********************************************************************
+ *           load_start_exe
+ *
+ * Load start.exe as main image.
+ */
+NTSTATUS load_start_exe( WCHAR **image, void **module )
+{
+    static const WCHAR startW[] = {'\\','?','?','\\','C',':','\\','w','i','n','d','o','w','s','\\',
+        's','y','s','t','e','m','3','2','\\','s','t','a','r','t','.','e','x','e',0};
+    UNICODE_STRING nt_name;
+    NTSTATUS status;
+    SIZE_T size;
+
+    init_unicode_string( &nt_name, startW );
+    status = find_builtin_dll( &nt_name, module, &size, &main_image_info, current_machine, FALSE );
+    if (status)
+    {
+        MESSAGE( "wine: failed to load start.exe: %x\n", status );
+        NtTerminateProcess( GetCurrentProcess(), status );
+    }
+    *image = malloc( sizeof(startW) );
+    memcpy( *image, startW, sizeof(startW) );
+    return status;
 }
 
 
@@ -1553,7 +1579,7 @@ static BOOL get_relocbase(caddr_t mapbase, caddr_t *relocbase)
 static void CDECL init_builtin_dll( void *module )
 {
 #ifdef HAVE_DLINFO
-    struct builtin_module *builtin;
+    void *handle = NULL;
     struct link_map *map;
     void (*init_func)(int, char **, char **) = NULL;
     void (**init_array)(int, char **, char **) = NULL;
@@ -1564,16 +1590,11 @@ static void CDECL init_builtin_dll( void *module )
     const Elf32_Dyn *dyn;
 #endif
 
-    LIST_FOR_EACH_ENTRY( builtin, &builtin_modules, struct builtin_module, entry )
-    {
-        if (builtin->module != module) continue;
-        if (!builtin->handle) break;
-        if (!dlinfo( builtin->handle, RTLD_DI_LINKMAP, &map )) goto found;
-        break;
-    }
-    return;
+    if (!(handle = get_builtin_so_handle( module ))) return;
+    if (dlinfo( handle, RTLD_DI_LINKMAP, &map )) map = NULL;
+    release_builtin_module( module );
+    if (!map) return;
 
-found:
     for (dyn = map->l_ld; dyn->d_tag; dyn++)
     {
         caddr_t relocbase = (caddr_t)map->l_addr;
@@ -1613,26 +1634,16 @@ static void load_ntdll(void)
     SECTION_IMAGE_INFORMATION info;
     OBJECT_ATTRIBUTES attr;
     UNICODE_STRING str;
-    HANDLE mapping;
-    struct stat st;
     void *module;
+    SIZE_T size = 0;
     char *name = build_path( dll_dir, "ntdll.dll.so" );
 
-    str.Buffer = path;
-    str.Length = sizeof(path) - sizeof(WCHAR);
-    str.MaximumLength = sizeof(path);
+    init_unicode_string( &str, path );
     InitializeObjectAttributes( &attr, &str, 0, 0, NULL );
     name[strlen(name) - 3] = 0;  /* remove .so */
-    status = open_builtin_file( name, &attr, &mapping, &module, &info, &st, FALSE );
-    if (!status && !module)
-    {
-        SIZE_T len = 0;
-        status = NtMapViewOfSection( mapping, NtCurrentProcess(), &module, 0, 0, NULL, &len,
-                                     ViewShare, 0, PAGE_EXECUTE_READ );
-        if (status == STATUS_IMAGE_NOT_AT_BASE) relocate_ntdll( module );
-        status = add_builtin_module( module, NULL, &st );
-    }
-    if (status) fatal_error( "failed to load %s error %x\n", name, status );
+    status = open_builtin_file( name, &attr, &module, &size, &info, current_machine, FALSE );
+    if (status == STATUS_IMAGE_NOT_AT_BASE) relocate_ntdll( module );
+    else if (status) fatal_error( "failed to load %s error %x\n", name, status );
     free( name );
     load_ntdll_functions( module );
     ntdll_module = module;
@@ -1715,13 +1726,10 @@ static struct unix_funcs unix_funcs =
     ntdll_sin,
     ntdll_sqrt,
     ntdll_tan,
-    get_unix_codepage_data,
-    get_locales,
     virtual_release_address_space,
     load_so_dll,
-    load_builtin_dll,
-    unload_builtin_dll,
     init_builtin_dll,
+    init_unix_lib,
     unwind_builtin_dll,
     __wine_dbg_get_channel_flags,
     __wine_dbg_strdup,
@@ -1736,6 +1744,7 @@ static struct unix_funcs unix_funcs =
 static void start_main_thread(void)
 {
     NTSTATUS status;
+    INITIAL_TEB stack;
     TEB *teb = virtual_alloc_first_teb();
 
     signal_init_threading();
@@ -1748,11 +1757,19 @@ static void start_main_thread(void)
     syscall_dispatcher = signal_init_syscalls();
     init_files();
     init_startup_info();
+    virtual_alloc_thread_stack( &stack, 0, 0, NULL );
+    teb->Tib.StackBase = stack.StackBase;
+    teb->Tib.StackLimit = stack.StackLimit;
+    teb->DeallocationStack = stack.DeallocationStack;
     NtCreateKeyedEvent( &keyed_event, GENERIC_READ | GENERIC_WRITE, NULL, 0 );
     load_ntdll();
     load_libwine();
     status = p__wine_set_unix_funcs( NTDLL_UNIXLIB_VERSION, &unix_funcs );
-    if (status) exec_process( status );
+    if (status == STATUS_REVISION_MISMATCH)
+    {
+        ERR( "ntdll library version mismatch\n" );
+        NtTerminateProcess( GetCurrentProcess(), status );
+    }
     server_init_process_done();
 }
 
@@ -2039,7 +2056,8 @@ static void check_command_line( int argc, char *argv[] )
     }
     if (!strcmp( argv[1], "--version" ))
     {
-        printf( "%s\n", wine_get_build_id() );
+        extern const char wine_build[];
+        printf( "%s\n", wine_build );
         exit(0);
     }
 }
@@ -2064,7 +2082,7 @@ void __wine_main( int argc, char *argv[], char *envp[] )
 
             memcpy( new_argv + 1, argv, (argc + 1) * sizeof(*argv) );
             putenv( noexec );
-            loader_exec( argv0, new_argv, client_cpu );
+            loader_exec( argv0, new_argv, current_machine );
             fatal_error( "could not exec the wine loader\n" );
         }
     }
