@@ -461,6 +461,24 @@ enum i386_trap_code
 #endif
 };
 
+struct syscall_xsave
+{
+    union
+    {
+        XSAVE_FORMAT       xsave;
+        FLOATING_SAVE_AREA fsave;
+    } u;
+    struct
+    {
+        ULONG64 mask;
+        ULONG64 compaction_mask;
+        ULONG64 reserved[6];
+        M128A   ymm_high[8];
+    } xstate;
+};
+
+C_ASSERT( sizeof(struct syscall_xsave) == 0x2c0 );
+
 struct syscall_frame
 {
     DWORD              eflags;  /* 00 */
@@ -502,6 +520,8 @@ C_ASSERT( offsetof( TEB, GdiTebBatch ) + offsetof( struct x86_thread_data, gs )
 C_ASSERT( offsetof( TEB, GdiTebBatch ) + offsetof( struct x86_thread_data, exit_frame ) == 0x1f4 );
 C_ASSERT( offsetof( TEB, GdiTebBatch ) + offsetof( struct x86_thread_data, syscall_frame ) == 0x1f8 );
 
+static void *syscall_dispatcher;
+
 static inline struct x86_thread_data *x86_thread_data(void)
 {
     return (struct x86_thread_data *)ntdll_get_thread_data()->cpu_data;
@@ -517,6 +537,11 @@ void set_syscall_frame(void *frame)
     x86_thread_data()->syscall_frame = frame;
 }
 
+static struct syscall_xsave *get_syscall_xsave( struct syscall_frame *frame )
+{
+    return (struct syscall_xsave *)((ULONG_PTR)((struct syscall_xsave *)frame - 1) & ~63);
+}
+
 static inline WORD get_cs(void) { WORD res; __asm__( "movw %%cs,%0" : "=r" (res) ); return res; }
 static inline WORD get_ds(void) { WORD res; __asm__( "movw %%ds,%0" : "=r" (res) ); return res; }
 static inline WORD get_fs(void) { WORD res; __asm__( "movw %%fs,%0" : "=r" (res) ); return res; }
@@ -688,73 +713,6 @@ static inline void save_fpu( CONTEXT *context )
 }
 
 
-/***********************************************************************
- *           save_fpux
- *
- * Save the thread FPU extended context.
- */
-static inline void save_fpux( CONTEXT *context )
-{
-    /* we have to enforce alignment by hand */
-    char buffer[sizeof(XSAVE_FORMAT) + 16];
-    XSAVE_FORMAT *state = (XSAVE_FORMAT *)(((ULONG_PTR)buffer + 15) & ~15);
-
-    context->ContextFlags |= CONTEXT_EXTENDED_REGISTERS;
-    __asm__ __volatile__( "fxsave %0" : "=m" (*state) );
-    memcpy( context->ExtendedRegisters, state, sizeof(*state) );
-}
-
-
-/***********************************************************************
- *           save_xstate
- *
- * Save the XState context
- */
-static inline NTSTATUS save_xstate( CONTEXT *context )
-{
-    CONTEXT_EX *context_ex = (CONTEXT_EX *)(context + 1);
-    DECLSPEC_ALIGN(64) struct
-    {
-        XSAVE_FORMAT xsave;
-        XSTATE xstate;
-    }
-    xsave_area;
-    XSTATE *xs;
-
-    if (!(cpu_info.FeatureSet & CPU_FEATURE_AVX) || !(xs = xstate_from_context( context )))
-        return STATUS_SUCCESS;
-
-    if (context_ex->XState.Length < offsetof(XSTATE, YmmContext)
-            || context_ex->XState.Length > sizeof(XSTATE))
-        return STATUS_INVALID_PARAMETER;
-
-    if (xstate_compaction_enabled)
-    {
-        /* xsavec doesn't use anything from the save area. */
-        __asm__ volatile( "xsavec %0" : "=m"(xsave_area)
-                : "a" ((unsigned int)(xs->CompactionMask & (1 << XSTATE_AVX))), "d" (0) );
-    }
-    else
-    {
-        /* xsave preserves those bits in the mask which are not in EDX:EAX, so zero it. */
-        xsave_area.xstate.Mask = xsave_area.xstate.CompactionMask = 0;
-        __asm__ volatile( "xsave %0" : "=m"(xsave_area)
-                : "a" ((unsigned int)(xs->Mask & (1 << XSTATE_AVX))), "d" (0) );
-    }
-
-    memcpy(xs, &xsave_area.xstate, offsetof(XSTATE, YmmContext));
-    if (xs->Mask & (1 << XSTATE_AVX))
-    {
-        if (context_ex->XState.Length < sizeof(XSTATE))
-            return STATUS_BUFFER_OVERFLOW;
-
-        memcpy(&xs->YmmContext, &xsave_area.xstate.YmmContext, sizeof(xs->YmmContext));
-    }
-
-    return STATUS_SUCCESS;
-}
-
-
 /***********************************************************************
  *           restore_fpu
  *
@@ -769,57 +727,6 @@ static inline void restore_fpu( const CONTEXT *context )
 }
 
 
-/***********************************************************************
- *           restore_fpux
- *
- * Restore the FPU extended context
- */
-static inline void restore_fpux( const CONTEXT *context )
-{
-    /* we have to enforce alignment by hand */
-    char buffer[sizeof(XSAVE_FORMAT) + 16];
-    XSAVE_FORMAT *state = (XSAVE_FORMAT *)(((ULONG_PTR)buffer + 15) & ~15);
-
-    memcpy( state, context->ExtendedRegisters, sizeof(*state) );
-    /* reset the current interrupt status */
-    state->StatusWord &= state->ControlWord | 0xff80;
-    __asm__ __volatile__( "fxrstor %0" : : "m" (*state) );
-}
-
-/***********************************************************************
- *           restore_xstate
- *
- * Restore the XState context
- */
-static inline void restore_xstate( const CONTEXT *context )
-{
-    XSAVE_FORMAT *xrstor_base;
-    XSTATE *xs;
-
-    if (!(cpu_info.FeatureSet & CPU_FEATURE_AVX) || !(xs = xstate_from_context( context )))
-        return;
-
-    xrstor_base = (XSAVE_FORMAT *)xs - 1;
-
-    if (!(xs->CompactionMask & ((ULONG64)1 << 63)))
-    {
-        /* Non-compacted xrstor will load Mxcsr regardless of the specified mask. Loading garbage there
-         * may lead to fault. FPUX state should be restored by now, so we can reuse some space in
-         * ExtendedRegisters. */
-        XSAVE_FORMAT *fpux = (XSAVE_FORMAT *)context->ExtendedRegisters;
-        DWORD mxcsr, mxcsr_mask;
-
-        mxcsr = fpux->MxCsr;
-        mxcsr_mask = fpux->MxCsr_Mask;
-
-        assert( (void *)&xrstor_base->MxCsr > (void *)context->ExtendedRegisters );
-        xrstor_base->MxCsr = mxcsr;
-        xrstor_base->MxCsr_Mask = mxcsr_mask;
-    }
-    __asm__ volatile( "xrstor %0" : : "m"(*xrstor_base), "a" (4), "d" (0) );
-}
-
-
 /***********************************************************************
  *           fpux_to_fpu
  *
@@ -866,6 +773,32 @@ static void fpux_to_fpu( FLOATING_SAVE_AREA *fpu, const XSAVE_FORMAT *fpux )
 }
 
 
+/***********************************************************************
+ *           fpu_to_fpux
+ *
+ * Fill extended FPU context from standard one.
+ */
+static void fpu_to_fpux( XSAVE_FORMAT *fpux, const FLOATING_SAVE_AREA *fpu )
+{
+    unsigned int i;
+
+    fpux->ControlWord   = fpu->ControlWord;
+    fpux->StatusWord    = fpu->StatusWord;
+    fpux->ErrorOffset   = fpu->ErrorOffset;
+    fpux->ErrorSelector = fpu->ErrorSelector;
+    fpux->ErrorOpcode   = fpu->ErrorSelector >> 16;
+    fpux->DataOffset    = fpu->DataOffset;
+    fpux->DataSelector  = fpu->DataSelector;
+    fpux->TagWord       = 0;
+    for (i = 0; i < 8; i++)
+    {
+        if (((fpu->TagWord >> (i * 2)) & 3) != 3)
+            fpux->TagWord |= 1 << i;
+        memcpy( &fpux->FloatRegisters[i], &fpu->RegisterArea[10 * i], 10 );
+    }
+}
+
+
 /***********************************************************************
  *           save_context
  *
@@ -1040,6 +973,20 @@ __ASM_GLOBAL_FUNC( set_full_cpu_context,
  */
 void signal_restore_full_cpu_context(void)
 {
+    struct syscall_xsave *xsave = get_syscall_xsave( get_syscall_frame() );
+
+    if (cpu_info.FeatureSet & CPU_FEATURE_XSAVE)
+    {
+        __asm__ volatile( "xrstor %0" : : "m"(*xsave), "a" (7), "d" (0) );
+    }
+    else if (cpu_info.FeatureSet & CPU_FEATURE_FXSR)
+    {
+        __asm__ volatile( "fxrstor %0" : : "m"(xsave->u.xsave) );
+    }
+    else
+    {
+        __asm__ volatile( "frstor %0; fwait" : : "m" (xsave->u.fsave) );
+    }
     set_full_cpu_context();
 }
 
@@ -1219,6 +1166,7 @@ NTSTATUS WINAPI NtSetContextThread( HANDLE handle, const CONTEXT *context )
     struct syscall_frame *frame = x86_thread_data()->syscall_frame;
     DWORD flags = context->ContextFlags & ~CONTEXT_i386;
     BOOL self = (handle == GetCurrentThread());
+    XSTATE *xs;
 
     /* debug registers require a server call */
     if (self && (flags & CONTEXT_DEBUG_REGISTERS))
@@ -1271,13 +1219,48 @@ NTSTATUS WINAPI NtSetContextThread( HANDLE handle, const CONTEXT *context )
         frame->fs = context->SegFs;
         frame->gs = context->SegGs;
     }
-    if (flags & CONTEXT_EXTENDED_REGISTERS) restore_fpux( context );
-    else if (flags & CONTEXT_FLOATING_POINT) restore_fpu( context );
+    if (flags & CONTEXT_EXTENDED_REGISTERS)
+    {
+        struct syscall_xsave *xsave = get_syscall_xsave( frame );
+        memcpy( &xsave->u.xsave, context->ExtendedRegisters, sizeof(xsave->u.xsave) );
+        /* reset the current interrupt status */
+        xsave->u.xsave.StatusWord &= xsave->u.xsave.ControlWord | 0xff80;
+        xsave->xstate.mask |= XSTATE_MASK_LEGACY;
+    }
+    else if (flags & CONTEXT_FLOATING_POINT)
+    {
+        struct syscall_xsave *xsave = get_syscall_xsave( frame );
+        if (cpu_info.FeatureSet & CPU_FEATURE_FXSR)
+        {
+            fpu_to_fpux( &xsave->u.xsave, &context->FloatSave );
+        }
+        else
+        {
+            xsave->u.fsave = context->FloatSave;
+        }
+        xsave->xstate.mask |= XSTATE_MASK_LEGACY_FLOATING_POINT;
+    }
+    if ((cpu_info.FeatureSet & CPU_FEATURE_AVX) && (xs = xstate_from_context( context )))
+    {
+        struct syscall_xsave *xsave = get_syscall_xsave( frame );
+        CONTEXT_EX *context_ex = (CONTEXT_EX *)(context + 1);
 
-    restore_xstate( context );
+        if (context_ex->XState.Length < offsetof(XSTATE, YmmContext)
+            || context_ex->XState.Length > sizeof(XSTATE))
+            return STATUS_INVALID_PARAMETER;
+
+        if (xs->Mask & XSTATE_MASK_GSSE)
+        {
+            if (context_ex->XState.Length < sizeof(XSTATE))
+                return STATUS_BUFFER_OVERFLOW;
+
+            xsave->xstate.mask |= XSTATE_MASK_GSSE;
+            memcpy( &xsave->xstate.ymm_high, &xs->YmmContext, sizeof(xsave->xstate.ymm_high) );
+        }
+        else
+            xsave->xstate.mask &= ~XSTATE_MASK_GSSE;
+    }
 
-    if (!(flags & CONTEXT_INTEGER)) frame->eax = STATUS_SUCCESS;
-    signal_restore_full_cpu_context();
     return STATUS_SUCCESS;
 }
 
@@ -1291,14 +1274,10 @@ NTSTATUS WINAPI NtSetContextThread( HANDLE handle, const CONTEXT *context )
  */
 NTSTATUS WINAPI NtGetContextThread( HANDLE handle, CONTEXT *context )
 {
-    NTSTATUS ret, xsave_status;
     struct syscall_frame *frame = x86_thread_data()->syscall_frame;
     DWORD needed_flags = context->ContextFlags & ~CONTEXT_i386;
     BOOL self = (handle == GetCurrentThread());
-
-    /* Save xstate before any calls which can potentially change volatile ymm registers.
-     * E. g., debug output will clobber ymm registers. */
-    xsave_status = self ? save_xstate( context ) : STATUS_SUCCESS;
+    NTSTATUS ret;
 
     /* debug registers require a server call */
     if (needed_flags & CONTEXT_DEBUG_REGISTERS) self = FALSE;
@@ -1315,6 +1294,8 @@ NTSTATUS WINAPI NtGetContextThread( HANDLE handle, CONTEXT *context )
 
     if (self)
     {
+        struct syscall_xsave *xsave = get_syscall_xsave( frame );
+        XSTATE *xstate;
         if (needed_flags & CONTEXT_INTEGER)
         {
             context->Eax = frame->eax;
@@ -1332,7 +1313,7 @@ NTSTATUS WINAPI NtGetContextThread( HANDLE handle, CONTEXT *context )
             context->Eip    = frame->eip;
             context->EFlags = frame->eflags;
             context->SegCs  = frame->cs;
-            context->SegSs  = frame->ds;
+            context->SegSs  = frame->ss;
             context->ContextFlags |= CONTEXT_CONTROL;
         }
         if (needed_flags & CONTEXT_SEGMENTS)
@@ -1343,8 +1324,57 @@ NTSTATUS WINAPI NtGetContextThread( HANDLE handle, CONTEXT *context )
             context->SegGs = frame->gs;
             context->ContextFlags |= CONTEXT_SEGMENTS;
         }
-        if (needed_flags & CONTEXT_FLOATING_POINT) save_fpu( context );
-        if (needed_flags & CONTEXT_EXTENDED_REGISTERS) save_fpux( context );
+        if (needed_flags & CONTEXT_FLOATING_POINT)
+        {
+            if (!(cpu_info.FeatureSet & CPU_FEATURE_FXSR))
+            {
+                context->FloatSave = xsave->u.fsave;
+            }
+            else if (!xstate_compaction_enabled ||
+                     (xsave->xstate.mask & XSTATE_MASK_LEGACY_FLOATING_POINT))
+            {
+                fpux_to_fpu( &context->FloatSave, &xsave->u.xsave );
+            }
+            else
+            {
+                memset( &context->FloatSave, 0, sizeof(context->FloatSave) );
+                context->FloatSave.ControlWord = 0x37f;
+            }
+            context->ContextFlags |= CONTEXT_FLOATING_POINT;
+        }
+        if (needed_flags & CONTEXT_EXTENDED_REGISTERS)
+        {
+            XSAVE_FORMAT *xs = (XSAVE_FORMAT *)context->ExtendedRegisters;
+
+            if (!xstate_compaction_enabled ||
+                (xsave->xstate.mask & XSTATE_MASK_LEGACY_FLOATING_POINT))
+            {
+                memcpy( xs, &xsave->u.xsave, FIELD_OFFSET( XSAVE_FORMAT, MxCsr ));
+                memcpy( xs->FloatRegisters, xsave->u.xsave.FloatRegisters,
+                        sizeof( xs->FloatRegisters ));
+            }
+            else
+            {
+                memset( xs, 0, FIELD_OFFSET( XSAVE_FORMAT, MxCsr ));
+                memset( xs->FloatRegisters, 0, sizeof( xs->FloatRegisters ));
+                xs->ControlWord = 0x37f;
+            }
+
+            if (!xstate_compaction_enabled || (xsave->xstate.mask & XSTATE_MASK_LEGACY_SSE))
+            {
+                memcpy( xs->XmmRegisters, xsave->u.xsave.XmmRegisters, sizeof( xs->XmmRegisters ));
+                xs->MxCsr      = xsave->u.xsave.MxCsr;
+                xs->MxCsr_Mask = xsave->u.xsave.MxCsr_Mask;
+            }
+            else
+            {
+                memset( xs->XmmRegisters, 0, sizeof( xs->XmmRegisters ));
+                xs->MxCsr      = 0x1f80;
+                xs->MxCsr_Mask = 0x2ffff;
+            }
+
+            context->ContextFlags |= CONTEXT_EXTENDED_REGISTERS;
+        }
         /* update the cached version of the debug registers */
         if (context->ContextFlags & (CONTEXT_DEBUG_REGISTERS & ~CONTEXT_i386))
         {
@@ -1355,6 +1385,28 @@ NTSTATUS WINAPI NtGetContextThread( HANDLE handle, CONTEXT *context )
             x86_thread_data()->dr6 = context->Dr6;
             x86_thread_data()->dr7 = context->Dr7;
         }
+        if ((cpu_info.FeatureSet & CPU_FEATURE_AVX) && (xstate = xstate_from_context( context )))
+        {
+            struct syscall_xsave *xsave = get_syscall_xsave( frame );
+            CONTEXT_EX *context_ex = (CONTEXT_EX *)(context + 1);
+            unsigned int mask;
+
+            if (context_ex->XState.Length < offsetof(XSTATE, YmmContext)
+                || context_ex->XState.Length > sizeof(XSTATE))
+                return STATUS_INVALID_PARAMETER;
+
+            mask = (xstate_compaction_enabled ? xstate->CompactionMask : xstate->Mask) & XSTATE_MASK_GSSE;
+            xstate->Mask = xsave->xstate.mask & mask;
+            xstate->CompactionMask = xstate_compaction_enabled ? (0x8000000000000000 | mask) : 0;
+            memset( xstate->Reserved, 0, sizeof(xstate->Reserved) );
+            if (xstate->Mask)
+            {
+                if (context_ex->XState.Length < sizeof(XSTATE))
+                    return STATUS_BUFFER_OVERFLOW;
+
+                memcpy( &xstate->YmmContext, xsave->xstate.ymm_high, sizeof(xsave->xstate.ymm_high) );
+            }
+        }
     }
 
     if (context->ContextFlags & (CONTEXT_INTEGER & ~CONTEXT_i386))
@@ -1370,7 +1422,7 @@ NTSTATUS WINAPI NtGetContextThread( HANDLE handle, CONTEXT *context )
         TRACE( "%p: dr0=%08x dr1=%08x dr2=%08x dr3=%08x dr6=%08x dr7=%08x\n", handle,
                context->Dr0, context->Dr1, context->Dr2, context->Dr3, context->Dr6, context->Dr7 );
 
-    return xsave_status;
+    return STATUS_SUCCESS;
 }
 
 
@@ -1782,16 +1834,10 @@ __ASM_GLOBAL_FUNC( call_user_apc_dispatcher,
 /***********************************************************************
  *           call_raise_user_exception_dispatcher
  */
-__ASM_GLOBAL_FUNC( call_raise_user_exception_dispatcher,
-                   "movl %fs:0x1f8,%eax\n\t"   /* x86_thread_data()->syscall_frame */
-                   "movl 0x1c(%eax),%ebx\n\t"  /* frame->ebx */
-                   "movl 0x28(%eax),%edi\n\t"  /* frame->edi */
-                   "movl 0x2c(%eax),%esi\n\t"  /* frame->esi */
-                   "movl 0x30(%eax),%ebp\n\t"  /* frame->ebp */
-                   "movl 4(%esp),%edx\n\t"     /* dispatcher */
-                   "movl $0,%fs:0x1f8\n\t"
-                   "leal 0x38(%eax),%esp\n\t"
-                   "jmp *%edx" )
+void WINAPI call_raise_user_exception_dispatcher( NTSTATUS (WINAPI *dispatcher)(void) )
+{
+    x86_thread_data()->syscall_frame->eip = (DWORD)dispatcher;
+}
 
 
 /***********************************************************************
@@ -2169,9 +2215,22 @@ static void usr1_handler( int signal, siginfo_t *siginfo, void *sigcontext )
     struct xcontext xcontext;
 
     init_handler( sigcontext );
-    save_context( &xcontext, sigcontext );
-    wait_suspend( &xcontext.c );
-    restore_context( &xcontext, sigcontext );
+    if (x86_thread_data()->syscall_frame)
+    {
+        DECLSPEC_ALIGN(64) XSTATE xs;
+        xcontext.c.ContextFlags = CONTEXT_FULL;
+        context_init_xstate( &xcontext.c, &xs );
+
+        NtGetContextThread( GetCurrentThread(), &xcontext.c );
+        wait_suspend( &xcontext.c );
+        NtSetContextThread( GetCurrentThread(), &xcontext.c );
+    }
+    else
+    {
+        save_context( &xcontext, sigcontext );
+        wait_suspend( &xcontext.c );
+        restore_context( &xcontext, sigcontext );
+    }
 }
 
 
@@ -2430,6 +2489,7 @@ NTSTATUS signal_alloc_thread( TEB *teb )
     }
     else thread_data->fs = gdt_fs_sel;
 
+    teb->WOW32Reserved = syscall_dispatcher;
     return STATUS_SUCCESS;
 }
 
@@ -2507,7 +2567,20 @@ void signal_init_process(void)
  */
 void *signal_init_syscalls(void)
 {
-    return __wine_syscall_dispatcher;
+    extern void __wine_syscall_dispatcher_fxsave(void) DECLSPEC_HIDDEN;
+    extern void __wine_syscall_dispatcher_xsave(void) DECLSPEC_HIDDEN;
+    extern void __wine_syscall_dispatcher_xsavec(void) DECLSPEC_HIDDEN;
+
+    if (xstate_compaction_enabled)
+        syscall_dispatcher = __wine_syscall_dispatcher_xsavec;
+    else if (cpu_info.FeatureSet & CPU_FEATURE_XSAVE)
+        syscall_dispatcher = __wine_syscall_dispatcher_xsave;
+    else if (cpu_info.FeatureSet & CPU_FEATURE_FXSR)
+        syscall_dispatcher = __wine_syscall_dispatcher_fxsave;
+    else
+        syscall_dispatcher = __wine_syscall_dispatcher;
+
+    return NtCurrentTeb()->WOW32Reserved = syscall_dispatcher;
 }
 
 
