@@ -90,7 +90,7 @@ static const struct object_ops mailslot_ops =
     default_unlink_name,       /* unlink_name */
     mailslot_open_file,        /* open_file */
     no_kernel_obj_list,        /* get_kernel_obj_list */
-    fd_close_handle,           /* close_handle */
+    no_close_handle,           /* close_handle */
     mailslot_destroy           /* destroy */
 };
 
@@ -148,7 +148,7 @@ static const struct object_ops mail_writer_ops =
     NULL,                       /* unlink_name */
     no_open_file,               /* open_file */
     no_kernel_obj_list,         /* get_kernel_obj_list */
-    fd_close_handle,            /* close_handle */
+    no_close_handle,            /* close_handle */
     mail_writer_destroy         /* destroy */
 };
 
@@ -240,7 +240,7 @@ static const struct object_ops mailslot_device_file_ops =
     NULL,                                   /* unlink_name */
     no_open_file,                           /* open_file */
     no_kernel_obj_list,                     /* get_kernel_obj_list */
-    fd_close_handle,                        /* close_handle */
+    no_close_handle,                        /* close_handle */
     mailslot_device_file_destroy            /* destroy */
 };
 
