@@ -503,7 +503,7 @@ static void process_sigkill( void *private )
 static void start_sigkill_timer( struct process *process )
 {
     grab_object( process );
-    if (process->unix_pid != -1 && process->msg_fd)
+    if (process->unix_pid != -1)
         process->sigkill_timeout = add_timeout_user( -TICKS_PER_SEC, process_sigkill, process );
     else
         process_died( process );
@@ -536,6 +536,8 @@ struct process *create_process( int fd, struct process *parent, int inherit_all,
     process->is_system       = 0;
     process->debug_children  = 1;
     process->is_terminating  = 0;
+    process->imagelen        = 0;
+    process->image           = NULL;
     process->job             = NULL;
     process->console         = NULL;
     process->startup_state   = STARTUP_IN_PROGRESS;
@@ -559,7 +561,6 @@ struct process *create_process( int fd, struct process *parent, int inherit_all,
     list_init( &process->rawinput_devices );
 
     process->end_time = 0;
-    list_add_tail( &process_list, &process->entry );
 
     if (sd && !default_set_sd( &process->obj, sd, OWNER_SECURITY_INFORMATION | GROUP_SECURITY_INFORMATION |
                                DACL_SECURITY_INFORMATION | SACL_SECURITY_INFORMATION ))
@@ -646,11 +647,11 @@ static void process_destroy( struct object *obj )
     }
     if (process->console) release_object( process->console );
     if (process->msg_fd) release_object( process->msg_fd );
-    list_remove( &process->entry );
     if (process->idle_event) release_object( process->idle_event );
     if (process->id) free_ptid( process->id );
     if (process->token) release_object( process->token );
     free( process->dir_cache );
+    free( process->image );
 }
 
 /* dump a process on stdout for debugging purposes */
@@ -804,15 +805,11 @@ restart:
 /* kill all processes */
 static void kill_all_processes(void)
 {
-    for (;;)
-    {
-        struct process *process;
+    struct list *ptr;
 
-        LIST_FOR_EACH_ENTRY( process, &process_list, struct process, entry )
-        {
-            if (process->running_threads) break;
-        }
-        if (&process->entry == &process_list) break;  /* no process found */
+    while ((ptr = list_head( &process_list )))
+    {
+        struct process *process = LIST_ENTRY( ptr, struct process, entry );
         terminate_process( process, NULL, 1 );
     }
 }
@@ -828,7 +825,6 @@ void kill_console_processes( struct thread *renderer, int exit_code )
         LIST_FOR_EACH_ENTRY( process, &process_list, struct process, entry )
         {
             if (process == renderer->process) continue;
-            if (!process->running_threads) continue;
             if (process->console && console_get_renderer( process->console ) == renderer) break;
         }
         if (&process->entry == &process_list) break;  /* no process found */
@@ -846,8 +842,8 @@ static void process_killed( struct process *process )
     if (!process->is_system) close_process_desktop( process );
     process->winstation = 0;
     process->desktop = 0;
-    close_process_handles( process );
     cancel_process_asyncs( process );
+    close_process_handles( process );
     if (process->idle_event) release_object( process->idle_event );
     process->idle_event = NULL;
     assert( !process->console );
@@ -875,6 +871,7 @@ void add_process_thread( struct process *process, struct thread *thread )
     list_add_tail( &process->thread_list, &thread->proc_entry );
     if (!process->running_threads++)
     {
+        list_add_tail( &process_list, &process->entry );
         running_processes++;
         if (!process->is_system)
         {
@@ -901,6 +898,7 @@ void remove_process_thread( struct process *process, struct thread *thread )
         /* we have removed the last running thread, exit the process */
         process->exit_code = thread->exit_code;
         generate_debug_event( thread, DbgExitProcessStateChange, process );
+        list_remove( &process->entry );
         process_killed( process );
     }
     else generate_debug_event( thread, DbgExitThreadStateChange, thread );
@@ -947,13 +945,7 @@ void kill_process( struct process *process, int violent_death )
         process->msg_fd = NULL;
     }
 
-    if (process->sigkill_timeout)  /* already waiting for it to die */
-    {
-        remove_timeout_user( process->sigkill_timeout );
-        process->sigkill_timeout = NULL;
-        process_died( process );
-        return;
-    }
+    if (process->sigkill_timeout) return;  /* already waiting for it to die */
 
     if (violent_death) terminate_process( process, NULL, 1 );
     else
@@ -979,10 +971,8 @@ void detach_debugged_processes( struct debug_obj *debug_obj, int exit_code )
 
         /* find the first process being debugged by 'debugger' and still running */
         LIST_FOR_EACH_ENTRY( process, &process_list, struct process, entry )
-        {
-            if (!process->running_threads) continue;
             if (process->debug_obj == debug_obj) break;
-        }
+
         if (&process->entry == &process_list) break;  /* no process found */
         if (exit_code)
         {
@@ -1216,39 +1206,6 @@ DECL_HANDLER(new_process)
     release_object( info );
 }
 
-/* execute a new process, replacing the existing one */
-DECL_HANDLER(exec_process)
-{
-    struct process *process;
-    int socket_fd = thread_get_inflight_fd( current, req->socket_fd );
-
-    if (socket_fd == -1)
-    {
-        set_error( STATUS_INVALID_PARAMETER );
-        return;
-    }
-    if (fcntl( socket_fd, F_SETFL, O_NONBLOCK ) == -1)
-    {
-        set_error( STATUS_INVALID_HANDLE );
-        close( socket_fd );
-        return;
-    }
-    if (shutdown_stage)
-    {
-        set_error( STATUS_SHUTDOWN_IN_PROGRESS );
-        close( socket_fd );
-        return;
-    }
-    if (!is_cpu_supported( req->cpu ))
-    {
-        close( socket_fd );
-        return;
-    }
-    if (!(process = create_process( socket_fd, NULL, 0, NULL, NULL, NULL, 0, NULL ))) return;
-    create_thread( -1, process, NULL );
-    release_object( process );
-}
-
 /* Retrieve information about a newly started process */
 DECL_HANDLER(get_new_process_info)
 {
@@ -1365,8 +1322,12 @@ DECL_HANDLER(get_process_info)
             client_ptr_t base;
             const pe_image_info_t *info;
             struct memory_view *view = get_exe_view( process );
-            if (view && (info = get_view_image_info( view, &base )))
-                set_reply_data( info, min( sizeof(*info), get_reply_max_size() ));
+            if (view)
+            {
+                if ((info = get_view_image_info( view, &base )))
+                    set_reply_data( info, min( sizeof(*info), get_reply_max_size() ));
+            }
+            else set_error( STATUS_PROCESS_IS_TERMINATING );
         }
         release_object( process );
     }
@@ -1388,13 +1349,12 @@ DECL_HANDLER(get_process_debug_info)
 /* fetch the name of the process image */
 DECL_HANDLER(get_process_image_name)
 {
-    struct unicode_str name;
-    struct memory_view *view;
     struct process *process = get_process_from_handle( req->handle, PROCESS_QUERY_LIMITED_INFORMATION );
 
     if (!process) return;
-    if ((view = get_exe_view( process )) && get_view_nt_name( view, &name ))
+    if (process->image)
     {
+        struct unicode_str name = { process->image, process->imagelen };
         /* skip the \??\ prefix */
         if (req->win32 && name.len > 6 * sizeof(WCHAR) && name.str[5] == ':')
         {
@@ -1731,7 +1691,6 @@ DECL_HANDLER(list_processes)
 {
     struct process *process;
     struct thread *thread;
-    struct unicode_str nt_name;
     unsigned int pos = 0;
     char *buffer;
 
@@ -1740,10 +1699,8 @@ DECL_HANDLER(list_processes)
 
     LIST_FOR_EACH_ENTRY( process, &process_list, struct process, entry )
     {
-        struct memory_view *view = get_exe_view( process );
-        if (!view || !get_view_nt_name( view, &nt_name )) nt_name.len = 0;
         reply->info_size = (reply->info_size + 7) & ~7;
-        reply->info_size += sizeof(struct process_info) + nt_name.len;
+        reply->info_size += sizeof(struct process_info) + process->imagelen;
         reply->info_size = (reply->info_size + 7) & ~7;
         reply->info_size += process->running_threads * sizeof(struct thread_info);
         reply->process_count++;
@@ -1761,13 +1718,11 @@ DECL_HANDLER(list_processes)
     LIST_FOR_EACH_ENTRY( process, &process_list, struct process, entry )
     {
         struct process_info *process_info;
-        struct memory_view *view = get_exe_view( process );
 
         pos = (pos + 7) & ~7;
-        if (!view || !get_view_nt_name( view, &nt_name )) nt_name.len = 0;
         process_info = (struct process_info *)(buffer + pos);
         process_info->start_time = process->start_time;
-        process_info->name_len = nt_name.len;
+        process_info->name_len = process->imagelen;
         process_info->thread_count = process->running_threads;
         process_info->priority = process->priority;
         process_info->pid = process->id;
@@ -1775,8 +1730,8 @@ DECL_HANDLER(list_processes)
         process_info->handle_count = get_handle_table_count(process);
         process_info->unix_pid = process->unix_pid;
         pos += sizeof(*process_info);
-        memcpy( buffer + pos, nt_name.str, nt_name.len );
-        pos += nt_name.len;
+        memcpy( buffer + pos, process->image, process->imagelen );
+        pos += process->imagelen;
         pos = (pos + 7) & ~7;
         LIST_FOR_EACH_ENTRY( thread, &process->thread_list, struct thread, proc_entry )
         {
